
from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..models import (
    MergeRequest,
    MergeRequestBulkRequest,
    MergeRequestBulkResponse,
    MergeRequestQueryConfig,
    MergeRequestQueryResponse,
)


class MergeRequestService(BaseService):

    @cast_models
    def create_merge_request(
        self, request_body: MergeRequest = None
    ) -> Union[MergeRequest, str]:
        """You can use the Merge Request object to merge a development branch into the main branch.

         - To create a merge request, you need the branch IDs for the source and destination branches. The source branch is the branch you want to merge into the destination branch.

        - There are two merge request strategies you can choose from: OVERRIDE or CONFLICT_RESOLVE. An override merge automatically resolves any merge conflicts by prioritizing the branch specified in the `priorityBranch` field. If you choose the CONFLICT_RESOLVE strategy, you have the opportunity to review any conflicts and choose which version you want to keep.

        :param request_body: The request body., defaults to None
        :type request_body: MergeRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[MergeRequest, str]
        """

        Validator(MergeRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/MergeRequest",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return MergeRequest._unmap(response)
        if content == "application/xml":
            return MergeRequest._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_merge_request(self, id_: str) -> Union[MergeRequest, str]:
        """Retrieve more information about the recently performed merge.

         - The `resolution` parameter is generated from the original merge request and specifies either the source branch for the final content for the merge or the destination. It can have the following values:

          -  OVERRIDE: The source branch has taken priority
          -  KEEP_DESTINATION: The destination branch has taken priority
         - The `changeType` parameter is generated from a branch diff that is performed on merge and can be one of the following values:
          -  ADDED: A component was added to the source branch
          -  MODIFIED: A component was modified in the source branch
          -  DELETED: A component was deleted in the source branch

         After performing a merge request between two branches, you can use the merge request’s ID to retrieve more information about the recently performed merge. The following example shows a merge between two branches where something was deleted in the source branch:

         Send an HTTP GET to `https://api.boomi.com/api/rest/v1/{accountId}/MergeRequest/{mergeRequestId}` where `{accountId}` is the ID of the authenticating account and `{mergeRequestId}` is the ID of the merge request.

         You can also use the GET operation to view a user's current working branch:

         Send an HTTP GET to `https://api.boomi.com/api/rest/v1/{accountId}/UserAccountProperty/defaultWorkingBranch` where the `{accountId}` is the ID of the account for which you want to view the working branch.

        :param id_: ID of the merge request.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[MergeRequest, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/MergeRequest/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return MergeRequest._unmap(response)
        if content == "application/xml":
            return MergeRequest._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_merge_request(
        self, id_: str, request_body: MergeRequest = None
    ) -> Union[MergeRequest, str]:
        """update_merge_request

        :param request_body: The request body., defaults to None
        :type request_body: MergeRequest, optional
        :param id_: ID of the merge request.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[MergeRequest, str]
        """

        Validator(MergeRequest).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/MergeRequest/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return MergeRequest._unmap(response)
        if content == "application/xml":
            return MergeRequest._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_merge_request(self, id_: str) -> None:
        """- There are three actions you can choose from when executing a merge request:
         -  MERGE: Use to start or restart a merge request; the `stage` must be REVIEWING or FAILED_TO_MERGE
         -  REVERT: Use to revert a merge request; the `stage` must be MERGED or DELETED and `previousStage` is MERGED
         -  RETRY_DRAFTING: Use when the merge request `stage` is FAILED_TO_DRAFT or FAILED_TO_REDRAFT
         - If the merge is successful, the `stage` and/or `previousStage` might be in one of the following stages:
         -  DRAFTING: The merge request is in the queue.
        -  DRAFTED: The merge request is drafted for review.
        -  REVIEWING: The merge request is being reviewed.
        -  MERGING: The merge request is being processed.
        -  MERGED: The merge request has successfully completed.
        -  FAILED_TO_MERGE: The merge request failed to merge.
        -  NOT_EXIST: No previous merge request has been submitted. This stage is typically returned in the `previousStage` parameter.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/MergeRequest/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_merge_request(
        self, request_body: MergeRequestBulkRequest = None
    ) -> Union[MergeRequestBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: MergeRequestBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[MergeRequestBulkResponse, str]
        """

        Validator(MergeRequestBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/MergeRequest/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return MergeRequestBulkResponse._unmap(response)
        if content == "application/xml":
            return MergeRequestBulkResponse._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def execute_merge_request(
        self, id_: str, request_body: MergeRequest = None
    ) -> Union[MergeRequest, str]:
        """- These are the actions you can choose from when executing a merge request:
          -  MERGE: Use to start or restart a merge request; the stage must be REVIEWING or FAILED_TO_MERGE
          -  REVERT: Use to revert a merge request; the stage must be MERGED or DELETED and previousStage is MERGED
          -  RETRY_DRAFTING: Use when the merge request stage is FAILED_TO_DRAFT or FAILED_TO_REDRAFT
        - If the merge is successful, the `stage` and/or `previousStage` might be in one of the following stages:
          -  DRAFTING - The merge request is in the queue.
          -  DRAFTED - The merge request is drafted for review.
          -  REVIEWING - The merge request is being reviewed.
          * MERGING - The merge request is being processed.
          * MERGED - The merge request has successfully completed.
          * FAILED_TO_MERGE - The merge request failed to merge.

        :param request_body: The request body., defaults to None
        :type request_body: MergeRequest, optional
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[MergeRequest, str]
        """

        Validator(MergeRequest).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/MergeRequest/execute/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return MergeRequest._unmap(response)
        if content == "application/xml":
            return MergeRequest._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_merge_request(
        self, request_body: MergeRequestQueryConfig = None
    ) -> Union[MergeRequestQueryResponse, str]:
        """- You can query a branch to retrieve a list of all active merge request IDs.
          - You must include the destination or source branch as a parameter. Only EQUALS is allowed for these parameters.
         - Optional parameters include:
          -   `createdDate`
          -   `createdBy`
          -   `stage`
          -   `modifiedDate`
          -   `modifiedBy`
        -  You can use the `queryMore` request to return more than 100 results.

        For more information about query filters, refer to [Query filters](/api/platformapi#section/Introduction/Query-filters).

        :param request_body: The request body., defaults to None
        :type request_body: MergeRequestQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[MergeRequestQueryResponse, str]
        """

        Validator(MergeRequestQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/MergeRequest/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return MergeRequestQueryResponse._unmap(response)
        if content == "application/xml":
            return MergeRequestQueryResponse._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_merge_request(
        self, request_body: str
    ) -> Union[MergeRequestQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[MergeRequestQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/MergeRequest/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return MergeRequestQueryResponse._unmap(response)
        if content == "application/xml":
            return MergeRequestQueryResponse._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .packaged_component_simple_expression import PackagedComponentSimpleExpression
from .packaged_component_grouping_expression import PackagedComponentGroupingExpression


class PackagedComponentExpressionGuard(OneOfBaseModel):
    class_list = {
        "PackagedComponentSimpleExpression": PackagedComponentSimpleExpression,
        "PackagedComponentGroupingExpression": PackagedComponentGroupingExpression,
    }


PackagedComponentExpression = Union[
    PackagedComponentSimpleExpression, PackagedComponentGroupingExpression
]




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .x12_control_info import X12ControlInfo
from .x12_options import X12Options


@JsonMap({"x12_control_info": "X12ControlInfo", "x12_options": "X12Options"})
class X12PartnerInfo(BaseModel):
    """X12PartnerInfo

    :param x12_control_info: x12_control_info, defaults to None
    :type x12_control_info: X12ControlInfo, optional
    :param x12_options: x12_options, defaults to None
    :type x12_options: X12Options, optional
    """

    def __init__(
        self,
        x12_control_info: X12ControlInfo = SENTINEL,
        x12_options: X12Options = SENTINEL,
        **kwargs,
    ):
        """X12PartnerInfo

        :param x12_control_info: x12_control_info, defaults to None
        :type x12_control_info: X12ControlInfo, optional
        :param x12_options: x12_options, defaults to None
        :type x12_options: X12Options, optional
        """
        if x12_control_info is not SENTINEL:
            self.x12_control_info = self._define_object(
                x12_control_info, X12ControlInfo
            )
        if x12_options is not SENTINEL:
            self.x12_options = self._define_object(x12_options, X12Options)
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..process_schedules import ProcessSchedulesService
from ...models import (
    ProcessSchedules,
    ProcessSchedulesBulkResponse,
    ProcessSchedulesBulkRequest,
    ProcessSchedulesQueryResponse,
    ProcessSchedulesQueryConfig,
)


class ProcessSchedulesServiceAsync(ProcessSchedulesService):
    """
    Async Wrapper for ProcessSchedulesServiceAsync
    """

    def get_process_schedules(
        self, id_: str
    ) -> Awaitable[Union[ProcessSchedules, str]]:
        return to_async(super().get_process_schedules)(id_)

    def update_process_schedules(
        self, id_: str, request_body: ProcessSchedules = None
    ) -> Awaitable[Union[ProcessSchedules, str]]:
        return to_async(super().update_process_schedules)(id_, request_body)

    def bulk_process_schedules(
        self, request_body: ProcessSchedulesBulkRequest = None
    ) -> Awaitable[Union[ProcessSchedulesBulkResponse, str]]:
        return to_async(super().bulk_process_schedules)(request_body)

    def query_process_schedules(
        self, request_body: ProcessSchedulesQueryConfig = None
    ) -> Awaitable[Union[ProcessSchedulesQueryResponse, str]]:
        return to_async(super().query_process_schedules)(request_body)

    def query_more_process_schedules(
        self, request_body: str
    ) -> Awaitable[Union[ProcessSchedulesQueryResponse, str]]:
        return to_async(super().query_more_process_schedules)(request_body)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .atom_simple_expression import AtomSimpleExpression
from .atom_grouping_expression import AtomGroupingExpression


class AtomExpressionGuard(OneOfBaseModel):
    class_list = {
        "AtomSimpleExpression": AtomSimpleExpression,
        "AtomGroupingExpression": AtomGroupingExpression,
    }


AtomExpression = Union[AtomSimpleExpression, AtomGroupingExpression]




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .atom_disk_space_directory import AtomDiskSpaceDirectory


@JsonMap(
    {
        "disk_space_directory": "DiskSpaceDirectory",
        "quota_limit": "quotaLimit",
        "raw_quota_limit": "rawQuotaLimit",
        "raw_total_size": "rawTotalSize",
        "total_size": "totalSize",
    }
)
class AtomDiskSpace(BaseModel):
    """AtomDiskSpace

    :param disk_space_directory: disk_space_directory, defaults to None
    :type disk_space_directory: List[AtomDiskSpaceDirectory], optional
    :param quota_limit: The total amount of disk space available for consumption by this attachment., defaults to None
    :type quota_limit: str, optional
    :param raw_quota_limit: The total number of bytes available for consumption by this attachment., defaults to None
    :type raw_quota_limit: int, optional
    :param raw_total_size: The disk space in bytes that is already consumed by the given attachment., defaults to None
    :type raw_total_size: int, optional
    :param total_size: The size of disk space already consumed by the given attachment., defaults to None
    :type total_size: str, optional
    """

    def __init__(
        self,
        disk_space_directory: List[AtomDiskSpaceDirectory] = SENTINEL,
        quota_limit: str = SENTINEL,
        raw_quota_limit: int = SENTINEL,
        raw_total_size: int = SENTINEL,
        total_size: str = SENTINEL,
        **kwargs,
    ):
        """AtomDiskSpace

        :param disk_space_directory: disk_space_directory, defaults to None
        :type disk_space_directory: List[AtomDiskSpaceDirectory], optional
        :param quota_limit: The total amount of disk space available for consumption by this attachment., defaults to None
        :type quota_limit: str, optional
        :param raw_quota_limit: The total number of bytes available for consumption by this attachment., defaults to None
        :type raw_quota_limit: int, optional
        :param raw_total_size: The disk space in bytes that is already consumed by the given attachment., defaults to None
        :type raw_total_size: int, optional
        :param total_size: The size of disk space already consumed by the given attachment., defaults to None
        :type total_size: str, optional
        """
        if disk_space_directory is not SENTINEL:
            self.disk_space_directory = self._define_list(
                disk_space_directory, AtomDiskSpaceDirectory
            )
        if quota_limit is not SENTINEL:
            self.quota_limit = quota_limit
        if raw_quota_limit is not SENTINEL:
            self.raw_quota_limit = raw_quota_limit
        if raw_total_size is not SENTINEL:
            self.raw_total_size = raw_total_size
        if total_size is not SENTINEL:
            self.total_size = total_size
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..integration_pack_environment_attachment import (
    IntegrationPackEnvironmentAttachmentService,
)
from ...models import (
    IntegrationPackEnvironmentAttachment,
    IntegrationPackEnvironmentAttachmentQueryResponse,
    IntegrationPackEnvironmentAttachmentQueryConfig,
)


class IntegrationPackEnvironmentAttachmentServiceAsync(
    IntegrationPackEnvironmentAttachmentService
):
    """
    Async Wrapper for IntegrationPackEnvironmentAttachmentServiceAsync
    """

    def create_integration_pack_environment_attachment(
        self, request_body: IntegrationPackEnvironmentAttachment = None
    ) -> Awaitable[Union[IntegrationPackEnvironmentAttachment, str]]:
        return to_async(super().create_integration_pack_environment_attachment)(
            request_body
        )

    def query_integration_pack_environment_attachment(
        self, request_body: IntegrationPackEnvironmentAttachmentQueryConfig = None
    ) -> Awaitable[Union[IntegrationPackEnvironmentAttachmentQueryResponse, str]]:
        return to_async(super().query_integration_pack_environment_attachment)(
            request_body
        )

    def query_more_integration_pack_environment_attachment(
        self, request_body: str
    ) -> Awaitable[Union[IntegrationPackEnvironmentAttachmentQueryResponse, str]]:
        return to_async(super().query_more_integration_pack_environment_attachment)(
            request_body
        )

    def delete_integration_pack_environment_attachment(
        self, id_: str
    ) -> Awaitable[None]:
        return to_async(super().delete_integration_pack_environment_attachment)(id_)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .shared_communication_channel_component_simple_expression import (
    SharedCommunicationChannelComponentSimpleExpression,
)
from .shared_communication_channel_component_grouping_expression import (
    SharedCommunicationChannelComponentGroupingExpression,
)


class SharedCommunicationChannelComponentExpressionGuard(OneOfBaseModel):
    class_list = {
        "SharedCommunicationChannelComponentSimpleExpression": SharedCommunicationChannelComponentSimpleExpression,
        "SharedCommunicationChannelComponentGroupingExpression": SharedCommunicationChannelComponentGroupingExpression,
    }


SharedCommunicationChannelComponentExpression = Union[
    SharedCommunicationChannelComponentSimpleExpression,
    SharedCommunicationChannelComponentGroupingExpression,
]




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .role_simple_expression import RoleSimpleExpression
from .role_grouping_expression import RoleGroupingExpression


class RoleExpressionGuard(OneOfBaseModel):
    class_list = {
        "RoleSimpleExpression": RoleSimpleExpression,
        "RoleGroupingExpression": RoleGroupingExpression,
    }


RoleExpression = Union[RoleSimpleExpression, RoleGroupingExpression]




from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "enable_http_request_handling": "EnableHTTPRequestHandling",
        "enable_https_request_handling": "EnableHTTPSRequestHandling",
        "allow_credentials": "allowCredentials",
        "allow_methods": "allowMethods",
        "allow_request_headers": "allowRequestHeaders",
        "allow_response_headers": "allowResponseHeaders",
        "cache_timeout": "cacheTimeout",
    }
)
class SharedWebServerCorsOrigin(BaseModel):
    """SharedWebServerCorsOrigin

    :param enable_http_request_handling: enable_http_request_handling, defaults to None
    :type enable_http_request_handling: bool, optional
    :param enable_https_request_handling: enable_https_request_handling, defaults to None
    :type enable_https_request_handling: bool, optional
    :param allow_credentials: allow_credentials, defaults to None
    :type allow_credentials: bool, optional
    :param allow_methods: allow_methods, defaults to None
    :type allow_methods: List[str], optional
    :param allow_request_headers: allow_request_headers, defaults to None
    :type allow_request_headers: List[str], optional
    :param allow_response_headers: allow_response_headers, defaults to None
    :type allow_response_headers: List[str], optional
    :param cache_timeout: cache_timeout, defaults to None
    :type cache_timeout: int, optional
    :param domain: domain
    :type domain: str
    :param ports: ports, defaults to None
    :type ports: List[int], optional
    """

    def __init__(
        self,
        domain: str,
        enable_http_request_handling: bool = SENTINEL,
        enable_https_request_handling: bool = SENTINEL,
        allow_credentials: bool = SENTINEL,
        allow_methods: List[str] = SENTINEL,
        allow_request_headers: List[str] = SENTINEL,
        allow_response_headers: List[str] = SENTINEL,
        cache_timeout: int = SENTINEL,
        ports: List[int] = SENTINEL,
        **kwargs
    ):
        """SharedWebServerCorsOrigin

        :param enable_http_request_handling: enable_http_request_handling, defaults to None
        :type enable_http_request_handling: bool, optional
        :param enable_https_request_handling: enable_https_request_handling, defaults to None
        :type enable_https_request_handling: bool, optional
        :param allow_credentials: allow_credentials, defaults to None
        :type allow_credentials: bool, optional
        :param allow_methods: allow_methods, defaults to None
        :type allow_methods: List[str], optional
        :param allow_request_headers: allow_request_headers, defaults to None
        :type allow_request_headers: List[str], optional
        :param allow_response_headers: allow_response_headers, defaults to None
        :type allow_response_headers: List[str], optional
        :param cache_timeout: cache_timeout, defaults to None
        :type cache_timeout: int, optional
        :param domain: domain
        :type domain: str
        :param ports: ports, defaults to None
        :type ports: List[int], optional
        """
        if enable_http_request_handling is not SENTINEL:
            self.enable_http_request_handling = enable_http_request_handling
        if enable_https_request_handling is not SENTINEL:
            self.enable_https_request_handling = enable_https_request_handling
        if allow_credentials is not SENTINEL:
            self.allow_credentials = allow_credentials
        if allow_methods is not SENTINEL:
            self.allow_methods = allow_methods
        if allow_request_headers is not SENTINEL:
            self.allow_request_headers = allow_request_headers
        if allow_response_headers is not SENTINEL:
            self.allow_response_headers = allow_response_headers
        if cache_timeout is not SENTINEL:
            self.cache_timeout = cache_timeout
        self.domain = domain
        if ports is not SENTINEL:
            self.ports = ports
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .deployed_expired_certificate_expression import (
    DeployedExpiredCertificateExpression,
    DeployedExpiredCertificateExpressionGuard,
)
from .deployed_expired_certificate_simple_expression import (
    DeployedExpiredCertificateSimpleExpression,
)
from .deployed_expired_certificate_grouping_expression import (
    DeployedExpiredCertificateGroupingExpression,
)


@JsonMap({})
class DeployedExpiredCertificateQueryConfigQueryFilter(BaseModel):
    """DeployedExpiredCertificateQueryConfigQueryFilter

    :param expression: expression
    :type expression: DeployedExpiredCertificateExpression
    """

    def __init__(self, expression: DeployedExpiredCertificateExpression, **kwargs):
        """DeployedExpiredCertificateQueryConfigQueryFilter

        :param expression: expression
        :type expression: DeployedExpiredCertificateExpression
        """
        self.expression = DeployedExpiredCertificateExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class DeployedExpiredCertificateQueryConfig(BaseModel):
    """DeployedExpiredCertificateQueryConfig

    :param query_filter: query_filter
    :type query_filter: DeployedExpiredCertificateQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: DeployedExpiredCertificateQueryConfigQueryFilter, **kwargs
    ):
        """DeployedExpiredCertificateQueryConfig

        :param query_filter: query_filter
        :type query_filter: DeployedExpiredCertificateQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, DeployedExpiredCertificateQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .sftp_proxy_settings import SftpProxySettings
from .sftpssh_options import SftpsshOptions


@JsonMap(
    {
        "host": "host",
        "password": "password",
        "port": "port",
        "sftp_proxy_settings": "SFTPProxySettings",
        "sftpssh_options": "SFTPSSHOptions",
        "use_default_settings": "useDefaultSettings",
        "user": "user",
    }
)
class SftpSettings(BaseModel):
    """SftpSettings

    :param sftp_proxy_settings: sftp_proxy_settings
    :type sftp_proxy_settings: SftpProxySettings
    :param sftpssh_options: sftpssh_options
    :type sftpssh_options: SftpsshOptions
    :param host: host
    :type host: str
    :param password: password
    :type password: str
    :param port: port
    :type port: int
    :param use_default_settings: use_default_settings, defaults to None
    :type use_default_settings: bool, optional
    :param user: user
    :type user: str
    """

    def __init__(
        self,
        sftp_proxy_settings: SftpProxySettings,
        sftpssh_options: SftpsshOptions,
        host: str,
        password: str,
        port: int,
        user: str,
        use_default_settings: bool = SENTINEL,
        **kwargs,
    ):
        """SftpSettings

        :param sftp_proxy_settings: sftp_proxy_settings
        :type sftp_proxy_settings: SftpProxySettings
        :param sftpssh_options: sftpssh_options
        :type sftpssh_options: SftpsshOptions
        :param host: host
        :type host: str
        :param password: password
        :type password: str
        :param port: port
        :type port: int
        :param use_default_settings: use_default_settings, defaults to None
        :type use_default_settings: bool, optional
        :param user: user
        :type user: str
        """
        self.sftp_proxy_settings = self._define_object(
            sftp_proxy_settings, SftpProxySettings
        )
        self.sftpssh_options = self._define_object(sftpssh_options, SftpsshOptions)
        self.host = host
        self.password = password
        self.port = port
        if use_default_settings is not SENTINEL:
            self.use_default_settings = use_default_settings
        self.user = user
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .custom_tracked_field_simple_expression import CustomTrackedFieldSimpleExpression
from .custom_tracked_field_grouping_expression import (
    CustomTrackedFieldGroupingExpression,
)


class CustomTrackedFieldExpressionGuard(OneOfBaseModel):
    class_list = {
        "CustomTrackedFieldSimpleExpression": CustomTrackedFieldSimpleExpression,
        "CustomTrackedFieldGroupingExpression": CustomTrackedFieldGroupingExpression,
    }


CustomTrackedFieldExpression = Union[
    CustomTrackedFieldSimpleExpression, CustomTrackedFieldGroupingExpression
]




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..models import (
    PackagedComponentManifest,
    PackagedComponentManifestBulkRequest,
    PackagedComponentManifestBulkResponse,
)


class PackagedComponentManifestService(BaseService):

    @cast_models
    def get_packaged_component_manifest(
        self, package_id: str
    ) -> Union[PackagedComponentManifest, str]:
        """Retrieve a list of the included components and their summary metadata for a single version of a packaged component.

        :param package_id: The ID of the packaged component.
        :type package_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PackagedComponentManifest, str]
        """

        Validator(str).validate(package_id)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PackagedComponentManifest/{{packageId}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("packageId", package_id)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PackagedComponentManifest._unmap(response)
        if content == "application/xml":
            return PackagedComponentManifest._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_packaged_component_manifest(
        self, request_body: PackagedComponentManifestBulkRequest = None
    ) -> Union[PackagedComponentManifestBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: PackagedComponentManifestBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PackagedComponentManifestBulkResponse, str]
        """

        Validator(PackagedComponentManifestBulkRequest).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PackagedComponentManifest/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PackagedComponentManifestBulkResponse._unmap(response)
        if content == "application/xml":
            return PackagedComponentManifestBulkResponse._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .oftp_partner_group_type import OftpPartnerGroupType


@JsonMap({"gateway_partner_group": "GatewayPartnerGroup", "use_gateway": "useGateway"})
class OftpListenOptionsInfo(BaseModel):
    """OftpListenOptionsInfo

    :param gateway_partner_group: gateway_partner_group
    :type gateway_partner_group: OftpPartnerGroupType
    :param operation: operation, defaults to None
    :type operation: str, optional
    :param use_gateway: use_gateway, defaults to None
    :type use_gateway: bool, optional
    """

    def __init__(
        self,
        gateway_partner_group: OftpPartnerGroupType,
        operation: str = SENTINEL,
        use_gateway: bool = SENTINEL,
        **kwargs,
    ):
        """OftpListenOptionsInfo

        :param gateway_partner_group: gateway_partner_group
        :type gateway_partner_group: OftpPartnerGroupType
        :param operation: operation, defaults to None
        :type operation: str, optional
        :param use_gateway: use_gateway, defaults to None
        :type use_gateway: bool, optional
        """
        self.gateway_partner_group = self._define_object(
            gateway_partner_group, OftpPartnerGroupType
        )
        if operation is not SENTINEL:
            self.operation = operation
        if use_gateway is not SENTINEL:
            self.use_gateway = use_gateway
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .edi_custom_connector_record_expression import (
    EdiCustomConnectorRecordExpression,
    EdiCustomConnectorRecordExpressionGuard,
)
from .edi_custom_connector_record_simple_expression import (
    EdiCustomConnectorRecordSimpleExpression,
)
from .edi_custom_connector_record_grouping_expression import (
    EdiCustomConnectorRecordGroupingExpression,
)


@JsonMap({})
class EdiCustomConnectorRecordQueryConfigQueryFilter(BaseModel):
    """EdiCustomConnectorRecordQueryConfigQueryFilter

    :param expression: expression
    :type expression: EdiCustomConnectorRecordExpression
    """

    def __init__(self, expression: EdiCustomConnectorRecordExpression, **kwargs):
        """EdiCustomConnectorRecordQueryConfigQueryFilter

        :param expression: expression
        :type expression: EdiCustomConnectorRecordExpression
        """
        self.expression = EdiCustomConnectorRecordExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class EdiCustomConnectorRecordQueryConfig(BaseModel):
    """EdiCustomConnectorRecordQueryConfig

    :param query_filter: query_filter
    :type query_filter: EdiCustomConnectorRecordQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: EdiCustomConnectorRecordQueryConfigQueryFilter, **kwargs
    ):
        """EdiCustomConnectorRecordQueryConfig

        :param query_filter: query_filter
        :type query_filter: EdiCustomConnectorRecordQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, EdiCustomConnectorRecordQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class AccountSimpleExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar EQUALS: "EQUALS"
    :vartype EQUALS: str
    :cvar LIKE: "LIKE"
    :vartype LIKE: str
    :cvar NOTEQUALS: "NOT_EQUALS"
    :vartype NOTEQUALS: str
    :cvar ISNULL: "IS_NULL"
    :vartype ISNULL: str
    :cvar ISNOTNULL: "IS_NOT_NULL"
    :vartype ISNOTNULL: str
    :cvar BETWEEN: "BETWEEN"
    :vartype BETWEEN: str
    :cvar GREATERTHAN: "GREATER_THAN"
    :vartype GREATERTHAN: str
    :cvar GREATERTHANOREQUAL: "GREATER_THAN_OR_EQUAL"
    :vartype GREATERTHANOREQUAL: str
    :cvar LESSTHAN: "LESS_THAN"
    :vartype LESSTHAN: str
    :cvar LESSTHANOREQUAL: "LESS_THAN_OR_EQUAL"
    :vartype LESSTHANOREQUAL: str
    :cvar CONTAINS: "CONTAINS"
    :vartype CONTAINS: str
    :cvar NOTCONTAINS: "NOT_CONTAINS"
    :vartype NOTCONTAINS: str
    """

    EQUALS = "EQUALS"
    LIKE = "LIKE"
    NOTEQUALS = "NOT_EQUALS"
    ISNULL = "IS_NULL"
    ISNOTNULL = "IS_NOT_NULL"
    BETWEEN = "BETWEEN"
    GREATERTHAN = "GREATER_THAN"
    GREATERTHANOREQUAL = "GREATER_THAN_OR_EQUAL"
    LESSTHAN = "LESS_THAN"
    LESSTHANOREQUAL = "LESS_THAN_OR_EQUAL"
    CONTAINS = "CONTAINS"
    NOTCONTAINS = "NOT_CONTAINS"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, AccountSimpleExpressionOperator._member_map_.values()
            )
        )


class AccountSimpleExpressionProperty(Enum):
    """An enumeration representing different categories.

    :cvar ACCOUNTID: "ACCOUNTID"
    :vartype ACCOUNTID: str
    :cvar DATECREATED: "DATECREATED"
    :vartype DATECREATED: str
    :cvar EXPIRATIONDATE: "EXPIRATIONDATE"
    :vartype EXPIRATIONDATE: str
    :cvar NAME: "NAME"
    :vartype NAME: str
    :cvar STATUS: "STATUS"
    :vartype STATUS: str
    :cvar WIDGETACCOUNT: "WIDGETACCOUNT"
    :vartype WIDGETACCOUNT: str
    :cvar OVERDEPLOYED: "OVERDEPLOYED"
    :vartype OVERDEPLOYED: str
    """

    ACCOUNTID = "ACCOUNTID"
    DATECREATED = "DATECREATED"
    EXPIRATIONDATE = "EXPIRATIONDATE"
    NAME = "NAME"
    STATUS = "STATUS"
    WIDGETACCOUNT = "WIDGETACCOUNT"
    OVERDEPLOYED = "OVERDEPLOYED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, AccountSimpleExpressionProperty._member_map_.values()
            )
        )


@JsonMap({})
class AccountSimpleExpression(BaseModel):
    """AccountSimpleExpression

    :param argument: argument, defaults to None
    :type argument: List[str], optional
    :param operator: operator
    :type operator: AccountSimpleExpressionOperator
    :param property: property
    :type property: AccountSimpleExpressionProperty
    """

    def __init__(
        self,
        operator: AccountSimpleExpressionOperator,
        property: AccountSimpleExpressionProperty,
        argument: List[str] = SENTINEL,
        **kwargs
    ):
        """AccountSimpleExpression

        :param argument: argument, defaults to None
        :type argument: List[str], optional
        :param operator: operator
        :type operator: AccountSimpleExpressionOperator
        :param property: property
        :type property: AccountSimpleExpressionProperty
        """
        if argument is not SENTINEL:
            self.argument = argument
        self.operator = self._enum_matching(
            operator, AccountSimpleExpressionOperator.list(), "operator"
        )
        self.property = self._enum_matching(
            property, AccountSimpleExpressionProperty.list(), "property"
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .privileges import Privileges


@JsonMap(
    {
        "description": "Description",
        "privileges": "Privileges",
        "account_id": "accountId",
        "id_": "id",
        "parent_id": "parentId",
    }
)
class Role(BaseModel):
    """Role

    :param description: Description of the role, defaults to None
    :type description: str, optional
    :param privileges: One or more privileges assigned to the role., defaults to None
    :type privileges: Privileges, optional
    :param account_id: The account ID under which the role exists., defaults to None
    :type account_id: str, optional
    :param id_: The ID of the role., defaults to None
    :type id_: str, optional
    :param name: The name of the role., defaults to None
    :type name: str, optional
    :param parent_id: The ID of the role from which this role inherits privileges., defaults to None
    :type parent_id: str, optional
    """

    def __init__(
        self,
        description: str = SENTINEL,
        privileges: Privileges = SENTINEL,
        account_id: str = SENTINEL,
        id_: str = SENTINEL,
        name: str = SENTINEL,
        parent_id: str = SENTINEL,
        **kwargs,
    ):
        """Role

        :param description: Description of the role, defaults to None
        :type description: str, optional
        :param privileges: One or more privileges assigned to the role., defaults to None
        :type privileges: Privileges, optional
        :param account_id: The account ID under which the role exists., defaults to None
        :type account_id: str, optional
        :param id_: The ID of the role., defaults to None
        :type id_: str, optional
        :param name: The name of the role., defaults to None
        :type name: str, optional
        :param parent_id: The ID of the role from which this role inherits privileges., defaults to None
        :type parent_id: str, optional
        """
        if description is not SENTINEL:
            self.description = description
        if privileges is not SENTINEL:
            self.privileges = self._define_object(privileges, Privileges)
        if account_id is not SENTINEL:
            self.account_id = account_id
        if id_ is not SENTINEL:
            self.id_ = id_
        if name is not SENTINEL:
            self.name = name
        if parent_id is not SENTINEL:
            self.parent_id = parent_id
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..atom_purge import AtomPurgeService
from ...models import AtomPurge


class AtomPurgeServiceAsync(AtomPurgeService):
    """
    Async Wrapper for AtomPurgeServiceAsync
    """

    def update_atom_purge(
        self, id_: str, request_body: AtomPurge = None
    ) -> Awaitable[Union[AtomPurge, str]]:
        return to_async(super().update_atom_purge)(id_, request_body)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .component_reference_simple_expression import ComponentReferenceSimpleExpression
from .component_reference_grouping_expression import (
    ComponentReferenceGroupingExpression,
)


class ComponentReferenceExpressionGuard(OneOfBaseModel):
    class_list = {
        "ComponentReferenceSimpleExpression": ComponentReferenceSimpleExpression,
        "ComponentReferenceGroupingExpression": ComponentReferenceGroupingExpression,
    }


ComponentReferenceExpression = Union[
    ComponentReferenceSimpleExpression, ComponentReferenceGroupingExpression
]




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .cloud_expression import CloudExpression, CloudExpressionGuard
from .cloud_simple_expression import CloudSimpleExpression
from .cloud_grouping_expression import CloudGroupingExpression


@JsonMap({})
class CloudQueryConfigQueryFilter(BaseModel):
    """CloudQueryConfigQueryFilter

    :param expression: expression
    :type expression: CloudExpression
    """

    def __init__(self, expression: CloudExpression, **kwargs):
        """CloudQueryConfigQueryFilter

        :param expression: expression
        :type expression: CloudExpression
        """
        self.expression = CloudExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class CloudQueryConfig(BaseModel):
    """CloudQueryConfig

    :param query_filter: query_filter
    :type query_filter: CloudQueryConfigQueryFilter
    """

    def __init__(self, query_filter: CloudQueryConfigQueryFilter, **kwargs):
        """CloudQueryConfig

        :param query_filter: query_filter
        :type query_filter: CloudQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, CloudQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .processing_group_partner_document_route import ProcessingGroupPartnerDocumentRoute


class ProcessingGroupPartnerStandardRouteStandard(Enum):
    """An enumeration representing different categories.

    :cvar X12: "x12"
    :vartype X12: str
    :cvar EDIFACT: "edifact"
    :vartype EDIFACT: str
    :cvar HL7: "hl7"
    :vartype HL7: str
    :cvar CUSTOM: "custom"
    :vartype CUSTOM: str
    :cvar ROSETTANET: "rosettanet"
    :vartype ROSETTANET: str
    :cvar TRADACOMS: "tradacoms"
    :vartype TRADACOMS: str
    :cvar ODETTE: "odette"
    :vartype ODETTE: str
    """

    X12 = "x12"
    EDIFACT = "edifact"
    HL7 = "hl7"
    CUSTOM = "custom"
    ROSETTANET = "rosettanet"
    TRADACOMS = "tradacoms"
    ODETTE = "odette"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                ProcessingGroupPartnerStandardRouteStandard._member_map_.values(),
            )
        )


@JsonMap({"document_type_route": "DocumentTypeRoute", "process_id": "processId"})
class ProcessingGroupPartnerStandardRoute(BaseModel):
    """ProcessingGroupPartnerStandardRoute

    :param document_type_route: document_type_route, defaults to None
    :type document_type_route: List[ProcessingGroupPartnerDocumentRoute], optional
    :param process_id: process_id, defaults to None
    :type process_id: str, optional
    :param standard: standard, defaults to None
    :type standard: ProcessingGroupPartnerStandardRouteStandard, optional
    """

    def __init__(
        self,
        document_type_route: List[ProcessingGroupPartnerDocumentRoute] = SENTINEL,
        process_id: str = SENTINEL,
        standard: ProcessingGroupPartnerStandardRouteStandard = SENTINEL,
        **kwargs,
    ):
        """ProcessingGroupPartnerStandardRoute

        :param document_type_route: document_type_route, defaults to None
        :type document_type_route: List[ProcessingGroupPartnerDocumentRoute], optional
        :param process_id: process_id, defaults to None
        :type process_id: str, optional
        :param standard: standard, defaults to None
        :type standard: ProcessingGroupPartnerStandardRouteStandard, optional
        """
        if document_type_route is not SENTINEL:
            self.document_type_route = self._define_list(
                document_type_route, ProcessingGroupPartnerDocumentRoute
            )
        if process_id is not SENTINEL:
            self.process_id = process_id
        if standard is not SENTINEL:
            self.standard = self._enum_matching(
                standard, ProcessingGroupPartnerStandardRouteStandard.list(), "standard"
            )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .environment_extensions_expression import (
    EnvironmentExtensionsExpression,
    EnvironmentExtensionsExpressionGuard,
)
from .environment_extensions_simple_expression import (
    EnvironmentExtensionsSimpleExpression,
)
from .environment_extensions_grouping_expression import (
    EnvironmentExtensionsGroupingExpression,
)


@JsonMap({})
class EnvironmentExtensionsQueryConfigQueryFilter(BaseModel):
    """EnvironmentExtensionsQueryConfigQueryFilter

    :param expression: expression
    :type expression: EnvironmentExtensionsExpression
    """

    def __init__(self, expression: EnvironmentExtensionsExpression, **kwargs):
        """EnvironmentExtensionsQueryConfigQueryFilter

        :param expression: expression
        :type expression: EnvironmentExtensionsExpression
        """
        self.expression = EnvironmentExtensionsExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class EnvironmentExtensionsQueryConfig(BaseModel):
    """EnvironmentExtensionsQueryConfig

    :param query_filter: query_filter
    :type query_filter: EnvironmentExtensionsQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: EnvironmentExtensionsQueryConfigQueryFilter, **kwargs
    ):
        """EnvironmentExtensionsQueryConfig

        :param query_filter: query_filter
        :type query_filter: EnvironmentExtensionsQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, EnvironmentExtensionsQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {"dh_key_size_max1024": "dhKeySizeMax1024", "known_host_entry": "knownHostEntry"}
)
class SftpsshOptions(BaseModel):
    """SftpsshOptions

    :param dh_key_size_max1024: dh_key_size_max1024, defaults to None
    :type dh_key_size_max1024: bool, optional
    :param known_host_entry: known_host_entry
    :type known_host_entry: str
    :param sshkeyauth: sshkeyauth, defaults to None
    :type sshkeyauth: bool, optional
    :param sshkeypassword: sshkeypassword
    :type sshkeypassword: str
    :param sshkeypath: sshkeypath
    :type sshkeypath: str
    """

    def __init__(
        self,
        known_host_entry: str,
        sshkeypassword: str,
        sshkeypath: str,
        dh_key_size_max1024: bool = SENTINEL,
        sshkeyauth: bool = SENTINEL,
        **kwargs
    ):
        """SftpsshOptions

        :param dh_key_size_max1024: dh_key_size_max1024, defaults to None
        :type dh_key_size_max1024: bool, optional
        :param known_host_entry: known_host_entry
        :type known_host_entry: str
        :param sshkeyauth: sshkeyauth, defaults to None
        :type sshkeyauth: bool, optional
        :param sshkeypassword: sshkeypassword
        :type sshkeypassword: str
        :param sshkeypath: sshkeypath
        :type sshkeypath: str
        """
        if dh_key_size_max1024 is not SENTINEL:
            self.dh_key_size_max1024 = dh_key_size_max1024
        self.known_host_entry = known_host_entry
        if sshkeyauth is not SENTINEL:
            self.sshkeyauth = sshkeyauth
        self.sshkeypassword = sshkeypassword
        self.sshkeypath = sshkeypath
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class DeploymentListenerStatus(Enum):
    """An enumeration representing different categories.

    :cvar RUNNING: "RUNNING"
    :vartype RUNNING: str
    :cvar PAUSED: "PAUSED"
    :vartype PAUSED: str
    """

    RUNNING = "RUNNING"
    PAUSED = "PAUSED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, DeploymentListenerStatus._member_map_.values())
        )


@JsonMap(
    {
        "component_id": "componentId",
        "component_type": "componentType",
        "deployed_by": "deployedBy",
        "deployed_on": "deployedOn",
        "environment_id": "environmentId",
        "id_": "id",
        "listener_status": "listenerStatus",
        "process_id": "processId",
    }
)
class Deployment(BaseModel):
    """Deployment

    :param component_id: component_id
    :type component_id: str
    :param component_type: component_type
    :type component_type: str
    :param current: current, defaults to None
    :type current: bool, optional
    :param deployed_by: deployed_by
    :type deployed_by: str
    :param deployed_on: deployed_on
    :type deployed_on: str
    :param digest: digest
    :type digest: str
    :param environment_id: environment_id
    :type environment_id: str
    :param id_: id_
    :type id_: str
    :param listener_status: listener_status, defaults to None
    :type listener_status: DeploymentListenerStatus, optional
    :param message: message, defaults to None
    :type message: str, optional
    :param notes: notes
    :type notes: str
    :param process_id: process_id
    :type process_id: str
    :param version: version, defaults to None
    :type version: int, optional
    """

    def __init__(
        self,
        component_id: str,
        component_type: str,
        deployed_by: str,
        deployed_on: str,
        digest: str,
        environment_id: str,
        id_: str,
        notes: str,
        process_id: str,
        current: bool = SENTINEL,
        listener_status: DeploymentListenerStatus = SENTINEL,
        message: str = SENTINEL,
        version: int = SENTINEL,
        **kwargs
    ):
        """Deployment

        :param component_id: component_id
        :type component_id: str
        :param component_type: component_type
        :type component_type: str
        :param current: current, defaults to None
        :type current: bool, optional
        :param deployed_by: deployed_by
        :type deployed_by: str
        :param deployed_on: deployed_on
        :type deployed_on: str
        :param digest: digest
        :type digest: str
        :param environment_id: environment_id
        :type environment_id: str
        :param id_: id_
        :type id_: str
        :param listener_status: listener_status, defaults to None
        :type listener_status: DeploymentListenerStatus, optional
        :param message: message, defaults to None
        :type message: str, optional
        :param notes: notes
        :type notes: str
        :param process_id: process_id
        :type process_id: str
        :param version: version, defaults to None
        :type version: int, optional
        """
        self.component_id = component_id
        self.component_type = component_type
        if current is not SENTINEL:
            self.current = current
        self.deployed_by = deployed_by
        self.deployed_on = deployed_on
        self.digest = digest
        self.environment_id = environment_id
        self.id_ = id_
        if listener_status is not SENTINEL:
            self.listener_status = self._enum_matching(
                listener_status, DeploymentListenerStatus.list(), "listener_status"
            )
        if message is not SENTINEL:
            self.message = message
        self.notes = notes
        self.process_id = process_id
        if version is not SENTINEL:
            self.version = version
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class FtpSendOptionsFtpAction(Enum):
    """An enumeration representing different categories.

    :cvar ACTIONPUTRENAME: "actionputrename"
    :vartype ACTIONPUTRENAME: str
    :cvar ACTIONPUTAPPEND: "actionputappend"
    :vartype ACTIONPUTAPPEND: str
    :cvar ACTIONPUTERROR: "actionputerror"
    :vartype ACTIONPUTERROR: str
    :cvar ACTIONPUTOVERWRITE: "actionputoverwrite"
    :vartype ACTIONPUTOVERWRITE: str
    """

    ACTIONPUTRENAME = "actionputrename"
    ACTIONPUTAPPEND = "actionputappend"
    ACTIONPUTERROR = "actionputerror"
    ACTIONPUTOVERWRITE = "actionputoverwrite"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, FtpSendOptionsFtpAction._member_map_.values())
        )


class FtpSendOptionsTransferType(Enum):
    """An enumeration representing different categories.

    :cvar ASCII: "ascii"
    :vartype ASCII: str
    :cvar BINARY: "binary"
    :vartype BINARY: str
    """

    ASCII = "ascii"
    BINARY = "binary"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, FtpSendOptionsTransferType._member_map_.values())
        )


@JsonMap(
    {
        "ftp_action": "ftpAction",
        "move_to_directory": "moveToDirectory",
        "remote_directory": "remoteDirectory",
        "transfer_type": "transferType",
        "use_default_send_options": "useDefaultSendOptions",
    }
)
class FtpSendOptions(BaseModel):
    """FtpSendOptions

    :param ftp_action: ftp_action, defaults to None
    :type ftp_action: FtpSendOptionsFtpAction, optional
    :param move_to_directory: move_to_directory
    :type move_to_directory: str
    :param remote_directory: remote_directory
    :type remote_directory: str
    :param transfer_type: transfer_type, defaults to None
    :type transfer_type: FtpSendOptionsTransferType, optional
    :param use_default_send_options: use_default_send_options, defaults to None
    :type use_default_send_options: bool, optional
    """

    def __init__(
        self,
        move_to_directory: str,
        remote_directory: str,
        ftp_action: FtpSendOptionsFtpAction = SENTINEL,
        transfer_type: FtpSendOptionsTransferType = SENTINEL,
        use_default_send_options: bool = SENTINEL,
        **kwargs
    ):
        """FtpSendOptions

        :param ftp_action: ftp_action, defaults to None
        :type ftp_action: FtpSendOptionsFtpAction, optional
        :param move_to_directory: move_to_directory
        :type move_to_directory: str
        :param remote_directory: remote_directory
        :type remote_directory: str
        :param transfer_type: transfer_type, defaults to None
        :type transfer_type: FtpSendOptionsTransferType, optional
        :param use_default_send_options: use_default_send_options, defaults to None
        :type use_default_send_options: bool, optional
        """
        if ftp_action is not SENTINEL:
            self.ftp_action = self._enum_matching(
                ftp_action, FtpSendOptionsFtpAction.list(), "ftp_action"
            )
        self.move_to_directory = move_to_directory
        self.remote_directory = remote_directory
        if transfer_type is not SENTINEL:
            self.transfer_type = self._enum_matching(
                transfer_type, FtpSendOptionsTransferType.list(), "transfer_type"
            )
        if use_default_send_options is not SENTINEL:
            self.use_default_send_options = use_default_send_options
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..deployment import DeploymentService
from ...models import (
    Deployment,
    DeploymentBulkResponse,
    DeploymentBulkRequest,
    DeploymentQueryResponse,
    DeploymentQueryConfig,
    ProcessEnvironmentAttachmentQueryResponse,
    ProcessEnvironmentAttachmentQueryConfig,
)


class DeploymentServiceAsync(DeploymentService):
    """
    Async Wrapper for DeploymentServiceAsync
    """

    def create_deployment(
        self, request_body: Deployment = None
    ) -> Awaitable[Union[Deployment, str]]:
        return to_async(super().create_deployment)(request_body)

    def get_deployment(self, id_: str) -> Awaitable[Union[Deployment, str]]:
        return to_async(super().get_deployment)(id_)

    def bulk_deployment(
        self, request_body: DeploymentBulkRequest = None
    ) -> Awaitable[Union[DeploymentBulkResponse, str]]:
        return to_async(super().bulk_deployment)(request_body)

    def query_deployment(
        self, request_body: DeploymentQueryConfig = None
    ) -> Awaitable[Union[DeploymentQueryResponse, str]]:
        return to_async(super().query_deployment)(request_body)

    def query_more_deployment(
        self, request_body: str
    ) -> Awaitable[Union[DeploymentQueryResponse, str]]:
        return to_async(super().query_more_deployment)(request_body)

    def query_process_environment_attachment(
        self, request_body: ProcessEnvironmentAttachmentQueryConfig = None
    ) -> Awaitable[Union[ProcessEnvironmentAttachmentQueryResponse, str]]:
        return to_async(super().query_process_environment_attachment)(request_body)

    def query_more_process_environment_attachment(
        self, request_body: str
    ) -> Awaitable[Union[ProcessEnvironmentAttachmentQueryResponse, str]]:
        return to_async(super().query_more_process_environment_attachment)(request_body)

    def delete_process_environment_attachment(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_process_environment_attachment)(id_)




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..models import ChangeListenerStatusRequest


class ChangeListenerStatusService(BaseService):

    @cast_models
    def create_change_listener_status(
        self, request_body: ChangeListenerStatusRequest = None
    ) -> None:
        """You can use the changeListenerStatus operation to pause, resume, or restart listeners. A successful changeListenerStatus call returns an empty changeListenerStatusResponse to indicate acceptance of the request.

        :param request_body: The request body., defaults to None
        :type request_body: ChangeListenerStatusRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(ChangeListenerStatusRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ChangeListenerStatus",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, _ = self.send_request(serialized_request)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .odette_connector_record_expression import (
    OdetteConnectorRecordExpression,
    OdetteConnectorRecordExpressionGuard,
)
from .odette_connector_record_simple_expression import (
    OdetteConnectorRecordSimpleExpression,
)
from .odette_connector_record_grouping_expression import (
    OdetteConnectorRecordGroupingExpression,
)


@JsonMap({})
class OdetteConnectorRecordQueryConfigQueryFilter(BaseModel):
    """OdetteConnectorRecordQueryConfigQueryFilter

    :param expression: expression
    :type expression: OdetteConnectorRecordExpression
    """

    def __init__(self, expression: OdetteConnectorRecordExpression, **kwargs):
        """OdetteConnectorRecordQueryConfigQueryFilter

        :param expression: expression
        :type expression: OdetteConnectorRecordExpression
        """
        self.expression = OdetteConnectorRecordExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class OdetteConnectorRecordQueryConfig(BaseModel):
    """OdetteConnectorRecordQueryConfig

    :param query_filter: query_filter
    :type query_filter: OdetteConnectorRecordQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: OdetteConnectorRecordQueryConfigQueryFilter, **kwargs
    ):
        """OdetteConnectorRecordQueryConfig

        :param query_filter: query_filter
        :type query_filter: OdetteConnectorRecordQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, OdetteConnectorRecordQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    ProcessSchedules,
    ProcessSchedulesBulkRequest,
    ProcessSchedulesBulkResponse,
    ProcessSchedulesQueryConfig,
    ProcessSchedulesQueryResponse,
)


class ProcessSchedulesService(BaseService):

    @cast_models
    def get_process_schedules(self, id_: str) -> Union[ProcessSchedules, str]:
        """Retrieves the Process Schedules object with a specific conceptual ID.

         The ordinary GET operation retrieves the Process Schedules object with a specific conceptual ID. The bulk GET operation retrieves the Process Schedules objects with specific conceptual IDs to a maximum of 100. In addition, you can obtain conceptual IDs from the QUERY operation.

        :param id_: The object’s conceptual ID, which is synthesized from the process and Runtime IDs.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ProcessSchedules, str]
        """

        Validator(str).validate(id_)

        # ProcessSchedules ID is base64-encoded and should not be URL-encoded
        # when used as a path parameter. The API expects the raw base64 string.
        from urllib.parse import unquote

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ProcessSchedules/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("GET")
        )

        # Manually replace the ID in the URL without URL encoding
        serialized_request.url = serialized_request.url.replace("{id}", id_)

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ProcessSchedules._unmap(response)
        if content == "application/xml":
            return ProcessSchedules._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_process_schedules(
        self, id_: str, request_body: ProcessSchedules = None
    ) -> Union[ProcessSchedules, str]:
        """Clears and updates the process run schedules specified in the Process Schedules object with a specific ID. The body of the request must specify not only the conceptual object ID but also the Runtime and process IDs. You can obtain the object ID from a QUERY operation.

        A Process Schedules object exists for every deployed process. If you do not update the schedule, the object is empty and a run schedule is not in effect.

        >**Note:** Listener processes cannot be scheduled. If a listener process is referenced, the call will fail with a 400 status code.

        You must have the **Runtime Management** privilege and the **Scheduling** privilege to perform the UPDATE operation. If you have the **Runtime Management Read Access** privilege, you cannot update process run schedules.

        >**Note:** After you update run schedules for a process on a Runtime, those schedules appear in the **Scheduling** dialog using the Advanced (cron) syntax.

        You can additionally employ a Bulk UPDATE operation for the Process Schedules object. See related links for more information about performing a Bulk UPDATE operation.

        :param request_body: The request body., defaults to None
        :type request_body: ProcessSchedules, optional
        :param id_: The object’s conceptual ID, which is synthesized from the process and Runtime IDs.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ProcessSchedules, str]
        """

        Validator(ProcessSchedules).is_optional().validate(request_body)
        Validator(str).validate(id_)

        # ProcessSchedules ID is base64-encoded and should not be URL-encoded
        # when used as a path parameter. The API expects the raw base64 string.
        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ProcessSchedules/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        # Manually replace the ID in the URL without URL encoding
        serialized_request.url = serialized_request.url.replace("{id}", id_)

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ProcessSchedules._unmap(response)
        if content == "application/xml":
            return ProcessSchedules._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_process_schedules(
        self, request_body: ProcessSchedulesBulkRequest = None
    ) -> Union[ProcessSchedulesBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: ProcessSchedulesBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ProcessSchedulesBulkResponse, str]
        """

        Validator(ProcessSchedulesBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ProcessSchedules/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ProcessSchedulesBulkResponse._unmap(response)
        if content == "application/xml":
            return ProcessSchedulesBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_process_schedules(
        self, request_body: ProcessSchedulesQueryConfig = None
    ) -> Union[ProcessSchedulesQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: ProcessSchedulesQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ProcessSchedulesQueryResponse, str]
        """

        Validator(ProcessSchedulesQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ProcessSchedules/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ProcessSchedulesQueryResponse._unmap(response)
        if content == "application/xml":
            return ProcessSchedulesQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_process_schedules(
        self, request_body: str
    ) -> Union[ProcessSchedulesQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ProcessSchedulesQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ProcessSchedules/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ProcessSchedulesQueryResponse._unmap(response)
        if content == "application/xml":
            return ProcessSchedulesQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .account_group_expression import (
    AccountGroupExpression,
    AccountGroupExpressionGuard,
)
from .account_group_simple_expression import AccountGroupSimpleExpression
from .account_group_grouping_expression import AccountGroupGroupingExpression


@JsonMap({})
class AccountGroupQueryConfigQueryFilter(BaseModel):
    """AccountGroupQueryConfigQueryFilter

    :param expression: expression
    :type expression: AccountGroupExpression
    """

    def __init__(self, expression: AccountGroupExpression, **kwargs):
        """AccountGroupQueryConfigQueryFilter

        :param expression: expression
        :type expression: AccountGroupExpression
        """
        self.expression = AccountGroupExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class AccountGroupQueryConfig(BaseModel):
    """AccountGroupQueryConfig

    :param query_filter: query_filter
    :type query_filter: AccountGroupQueryConfigQueryFilter
    """

    def __init__(self, query_filter: AccountGroupQueryConfigQueryFilter, **kwargs):
        """AccountGroupQueryConfig

        :param query_filter: query_filter
        :type query_filter: AccountGroupQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, AccountGroupQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "created_by": "createdBy",
        "created_date": "createdDate",
        "deployment_id": "deploymentId",
        "id_": "id",
        "modified_by": "modifiedBy",
        "modified_date": "modifiedDate",
        "package_id": "packageId",
        "parent_id": "parentId",
    }
)
class Branch(BaseModel):
    """Branch

    :param created_by: The user who created the branch., defaults to None
    :type created_by: str, optional
    :param created_date: The date and time when the branch was created., defaults to None
    :type created_date: str, optional
    :param deleted: Whether the branch is deleted., defaults to None
    :type deleted: bool, optional
    :param deployment_id: deployment_id, defaults to None
    :type deployment_id: str, optional
    :param description: description, defaults to None
    :type description: str, optional
    :param id_: The ID of the branch., defaults to None
    :type id_: str, optional
    :param modified_by: The user who last modified the branch., defaults to None
    :type modified_by: str, optional
    :param modified_date: The date and time when the branch was updated., defaults to None
    :type modified_date: str, optional
    :param name: The name of the branch., defaults to None
    :type name: str, optional
    :param package_id: The ID of the packaged component from which the branch is created., defaults to None
    :type package_id: str, optional
    :param parent_id: The ID of the parent branch., defaults to None
    :type parent_id: str, optional
    :param ready: Whether the branch is ready to use., defaults to None
    :type ready: bool, optional
    :param stage: The branch status: CREATING, NORMAL (ready for use), or DELETING., defaults to None
    :type stage: str, optional
    """

    def __init__(
        self,
        created_by: str = SENTINEL,
        created_date: str = SENTINEL,
        deleted: bool = SENTINEL,
        deployment_id: str = SENTINEL,
        description: str = SENTINEL,
        id_: str = SENTINEL,
        modified_by: str = SENTINEL,
        modified_date: str = SENTINEL,
        name: str = SENTINEL,
        package_id: str = SENTINEL,
        parent_id: str = SENTINEL,
        ready: bool = SENTINEL,
        stage: str = SENTINEL,
        **kwargs
    ):
        """Branch

        :param created_by: The user who created the branch., defaults to None
        :type created_by: str, optional
        :param created_date: The date and time when the branch was created., defaults to None
        :type created_date: str, optional
        :param deleted: Whether the branch is deleted., defaults to None
        :type deleted: bool, optional
        :param deployment_id: deployment_id, defaults to None
        :type deployment_id: str, optional
        :param description: description, defaults to None
        :type description: str, optional
        :param id_: The ID of the branch., defaults to None
        :type id_: str, optional
        :param modified_by: The user who last modified the branch., defaults to None
        :type modified_by: str, optional
        :param modified_date: The date and time when the branch was updated., defaults to None
        :type modified_date: str, optional
        :param name: The name of the branch., defaults to None
        :type name: str, optional
        :param package_id: The ID of the packaged component from which the branch is created., defaults to None
        :type package_id: str, optional
        :param parent_id: The ID of the parent branch., defaults to None
        :type parent_id: str, optional
        :param ready: Whether the branch is ready to use., defaults to None
        :type ready: bool, optional
        :param stage: The branch status: CREATING, NORMAL (ready for use), or DELETING., defaults to None
        :type stage: str, optional
        """
        if created_by is not SENTINEL:
            self.created_by = created_by
        if created_date is not SENTINEL:
            self.created_date = created_date
        if deleted is not SENTINEL:
            self.deleted = deleted
        if deployment_id is not SENTINEL:
            self.deployment_id = deployment_id
        if description is not SENTINEL:
            self.description = description
        if id_ is not SENTINEL:
            self.id_ = id_
        if modified_by is not SENTINEL:
            self.modified_by = modified_by
        if modified_date is not SENTINEL:
            self.modified_date = modified_date
        if name is not SENTINEL:
            self.name = name
        if package_id is not SENTINEL:
            self.package_id = package_id
        if parent_id is not SENTINEL:
            self.parent_id = parent_id
        if ready is not SENTINEL:
            self.ready = ready
        if stage is not SENTINEL:
            self.stage = stage
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "component_type": "componentType",
        "environment_id": "environmentId",
        "id_": "id",
        "process_id": "processId",
    }
)
class ProcessEnvironmentAttachment(BaseModel):
    """ProcessEnvironmentAttachment

    :param component_type: component_type, defaults to None
    :type component_type: str, optional
    :param environment_id: environment_id, defaults to None
    :type environment_id: str, optional
    :param id_: id_, defaults to None
    :type id_: str, optional
    :param process_id: process_id, defaults to None
    :type process_id: str, optional
    """

    def __init__(
        self,
        component_type: str = SENTINEL,
        environment_id: str = SENTINEL,
        id_: str = SENTINEL,
        process_id: str = SENTINEL,
        **kwargs
    ):
        """ProcessEnvironmentAttachment

        :param component_type: component_type, defaults to None
        :type component_type: str, optional
        :param environment_id: environment_id, defaults to None
        :type environment_id: str, optional
        :param id_: id_, defaults to None
        :type id_: str, optional
        :param process_id: process_id, defaults to None
        :type process_id: str, optional
        """
        if component_type is not SENTINEL:
            self.component_type = component_type
        if environment_id is not SENTINEL:
            self.environment_id = environment_id
        if id_ is not SENTINEL:
            self.id_ = id_
        if process_id is not SENTINEL:
            self.process_id = process_id
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..models import (
    RosettaNetConnectorRecordQueryConfig,
    RosettaNetConnectorRecordQueryResponse,
)


class RosettaNetConnectorRecordService(BaseService):

    @cast_models
    def query_rosetta_net_connector_record(
        self, request_body: RosettaNetConnectorRecordQueryConfig = None
    ) -> Union[RosettaNetConnectorRecordQueryResponse, str]:
        """- To filter by a customField, use the format customFields/fieldName as the filter property where fieldName is the element name of the custom field in the EDIFACT Connector Record structure. To get a list of the available custom fields, refer to [Custom Tracked Field object](#tag/CustomTrackedField).
         - The STARTS_WITH operator accepts values that do not include spaces only.

         For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: RosettaNetConnectorRecordQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[RosettaNetConnectorRecordQueryResponse, str]
        """

        Validator(RosettaNetConnectorRecordQueryConfig).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/RosettaNetConnectorRecord/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return RosettaNetConnectorRecordQueryResponse._unmap(response)
        if content == "application/xml":
            return RosettaNetConnectorRecordQueryResponse._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_rosetta_net_connector_record(
        self, request_body: str
    ) -> Union[RosettaNetConnectorRecordQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[RosettaNetConnectorRecordQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/RosettaNetConnectorRecord/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return RosettaNetConnectorRecordQueryResponse._unmap(response)
        if content == "application/xml":
            return RosettaNetConnectorRecordQueryResponse._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .component_metadata_expression import (
    ComponentMetadataExpression,
    ComponentMetadataExpressionGuard,
)
from .component_metadata_simple_expression import ComponentMetadataSimpleExpression
from .component_metadata_grouping_expression import ComponentMetadataGroupingExpression


@JsonMap({})
class ComponentMetadataQueryConfigQueryFilter(BaseModel):
    """ComponentMetadataQueryConfigQueryFilter

    :param expression: expression
    :type expression: ComponentMetadataExpression
    """

    def __init__(self, expression: ComponentMetadataExpression, **kwargs):
        """ComponentMetadataQueryConfigQueryFilter

        :param expression: expression
        :type expression: ComponentMetadataExpression
        """
        self.expression = ComponentMetadataExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ComponentMetadataQueryConfig(BaseModel):
    """ComponentMetadataQueryConfig

    :param query_filter: query_filter
    :type query_filter: ComponentMetadataQueryConfigQueryFilter
    """

    def __init__(self, query_filter: ComponentMetadataQueryConfigQueryFilter, **kwargs):
        """ComponentMetadataQueryConfig

        :param query_filter: query_filter
        :type query_filter: ComponentMetadataQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ComponentMetadataQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "component_id": "componentId",
        "component_name": "componentName",
        "component_type": "componentType",
        "component_version": "componentVersion",
        "environment_map_extension_id": "environmentMapExtensionId",
    }
)
class EnvironmentMapExtensionExternalComponent(BaseModel):
    """EnvironmentMapExtensionExternalComponent

    :param component_id: The ID of the component. The component ID is available by querying the Component Metadata object., defaults to None
    :type component_id: str, optional
    :param component_name: The user-defined name given to the component., defaults to None
    :type component_name: str, optional
    :param component_type: The type of component. \>**Note:** Currently, this object retrieves Cross Reference Table type-components ('crossref') only., defaults to None
    :type component_type: str, optional
    :param component_version: component_version, defaults to None
    :type component_version: int, optional
    :param environment_map_extension_id: The ID of the environment map extension. To find the environmentMapExtensionId, you can first query the [Environment Map Extension object](/api/platformapi#tag/EnvironmentMapExtension) to retrieve a list of all available environment extensions for an account and copy the resultant environment map extension ID., defaults to None
    :type environment_map_extension_id: str, optional
    """

    def __init__(
        self,
        component_id: str = SENTINEL,
        component_name: str = SENTINEL,
        component_type: str = SENTINEL,
        component_version: int = SENTINEL,
        environment_map_extension_id: str = SENTINEL,
        **kwargs
    ):
        """EnvironmentMapExtensionExternalComponent

        :param component_id: The ID of the component. The component ID is available by querying the Component Metadata object., defaults to None
        :type component_id: str, optional
        :param component_name: The user-defined name given to the component., defaults to None
        :type component_name: str, optional
        :param component_type: The type of component. \>**Note:** Currently, this object retrieves Cross Reference Table type-components ('crossref') only., defaults to None
        :type component_type: str, optional
        :param component_version: component_version, defaults to None
        :type component_version: int, optional
        :param environment_map_extension_id: The ID of the environment map extension. To find the environmentMapExtensionId, you can first query the [Environment Map Extension object](/api/platformapi#tag/EnvironmentMapExtension) to retrieve a list of all available environment extensions for an account and copy the resultant environment map extension ID., defaults to None
        :type environment_map_extension_id: str, optional
        """
        if component_id is not SENTINEL:
            self.component_id = component_id
        if component_name is not SENTINEL:
            self.component_name = component_name
        if component_type is not SENTINEL:
            self.component_type = component_type
        if component_version is not SENTINEL:
            self.component_version = component_version
        if environment_map_extension_id is not SENTINEL:
            self.environment_map_extension_id = environment_map_extension_id
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from typing import List, TYPE_CHECKING
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL

if TYPE_CHECKING:
    from .environment_map_extension_external_component_expression import EnvironmentMapExtensionExternalComponentExpression, EnvironmentMapExtensionExternalComponentExpressionGuard
from .environment_map_extension_external_component_simple_expression import (
    EnvironmentMapExtensionExternalComponentSimpleExpression,
)

class EnvironmentMapExtensionExternalComponentGroupingExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar AND: "and"
    :vartype AND: str
    :cvar OR: "or"
    :vartype OR: str
    """

    AND = "and"
    OR = "or"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                EnvironmentMapExtensionExternalComponentGroupingExpressionOperator._member_map_.values(),
            )
        )

@JsonMap({"nested_expression": "nestedExpression"})
class EnvironmentMapExtensionExternalComponentGroupingExpression(BaseModel):
    """EnvironmentMapExtensionExternalComponentGroupingExpression

    :param nested_expression: nested_expression, defaults to None
    :type nested_expression: List[EnvironmentMapExtensionExternalComponentExpression], optional
    :param operator: operator
    :type operator: EnvironmentMapExtensionExternalComponentGroupingExpressionOperator
    """

    def __init__(
        self,
        operator: EnvironmentMapExtensionExternalComponentGroupingExpressionOperator,
        nested_expression: List[
            EnvironmentMapExtensionExternalComponentExpression
        ] = SENTINEL,
        **kwargs,
    ):
        """EnvironmentMapExtensionExternalComponentGroupingExpression

        :param nested_expression: nested_expression, defaults to None
        :type nested_expression: List[EnvironmentMapExtensionExternalComponentExpression], optional
        :param operator: operator
        :type operator: EnvironmentMapExtensionExternalComponentGroupingExpressionOperator
        """
        if nested_expression is not SENTINEL:
            self.nested_expression = self._define_list(
                nested_expression, EnvironmentMapExtensionExternalComponentExpression
            )
        self.operator = self._enum_matching(
            operator,
            EnvironmentMapExtensionExternalComponentGroupingExpressionOperator.list(),
            "operator",
        )
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    IntegrationPackEnvironmentAttachment,
    IntegrationPackEnvironmentAttachmentQueryConfig,
    IntegrationPackEnvironmentAttachmentQueryResponse,
)


class IntegrationPackEnvironmentAttachmentService(BaseService):

    @cast_models
    def create_integration_pack_environment_attachment(
        self, request_body: IntegrationPackEnvironmentAttachment = None
    ) -> Union[IntegrationPackEnvironmentAttachment, str]:
        """Attaches an integration pack instance having the specified ID to the environment having the specified ID.

        :param request_body: The request body., defaults to None
        :type request_body: IntegrationPackEnvironmentAttachment, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[IntegrationPackEnvironmentAttachment, str]
        """

        Validator(IntegrationPackEnvironmentAttachment).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/IntegrationPackEnvironmentAttachment",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return IntegrationPackEnvironmentAttachment._unmap(response)
        if content == "application/xml":
            return IntegrationPackEnvironmentAttachment._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_integration_pack_environment_attachment(
        self, request_body: IntegrationPackEnvironmentAttachmentQueryConfig = None
    ) -> Union[IntegrationPackEnvironmentAttachmentQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: IntegrationPackEnvironmentAttachmentQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[IntegrationPackEnvironmentAttachmentQueryResponse, str]
        """

        Validator(
            IntegrationPackEnvironmentAttachmentQueryConfig
        ).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/IntegrationPackEnvironmentAttachment/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return IntegrationPackEnvironmentAttachmentQueryResponse._unmap(response)
        if content == "application/xml":
            return IntegrationPackEnvironmentAttachmentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_integration_pack_environment_attachment(
        self, request_body: str
    ) -> Union[IntegrationPackEnvironmentAttachmentQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[IntegrationPackEnvironmentAttachmentQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/IntegrationPackEnvironmentAttachment/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return IntegrationPackEnvironmentAttachmentQueryResponse._unmap(response)
        if content == "application/xml":
            return IntegrationPackEnvironmentAttachmentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_integration_pack_environment_attachment(self, id_: str) -> None:
        """Detaches an integration pack instance from an environment where the conceptual Integration Pack Environment Attachment object ID specifies the attachment. If you successfully detach the integration pack instance from the environment, the response is `true`.

        :param id_: The conceptual Integration Pack Environment Attachment object ID
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/IntegrationPackEnvironmentAttachment/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .queue_record import QueueRecord


@JsonMap({"queue_record": "QueueRecord"})
class ListQueues(BaseModel):
    """ListQueues

    :param queue_record: queue_record, defaults to None
    :type queue_record: List[QueueRecord], optional
    """

    def __init__(self, queue_record: List[QueueRecord] = SENTINEL, **kwargs):
        """ListQueues

        :param queue_record: queue_record, defaults to None
        :type queue_record: List[QueueRecord], optional
        """
        if queue_record is not SENTINEL:
            self.queue_record = self._define_list(queue_record, QueueRecord)
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "namespace_id": "namespaceId",
        "universal_id": "universalId",
        "universal_id_type": "universalIdType",
    }
)
class HdType(BaseModel):
    """HdType

    :param namespace_id: namespace_id, defaults to None
    :type namespace_id: str, optional
    :param universal_id: universal_id, defaults to None
    :type universal_id: str, optional
    :param universal_id_type: universal_id_type, defaults to None
    :type universal_id_type: str, optional
    """

    def __init__(
        self,
        namespace_id: str = SENTINEL,
        universal_id: str = SENTINEL,
        universal_id_type: str = SENTINEL,
        **kwargs
    ):
        """HdType

        :param namespace_id: namespace_id, defaults to None
        :type namespace_id: str, optional
        :param universal_id: universal_id, defaults to None
        :type universal_id: str, optional
        :param universal_id_type: universal_id_type, defaults to None
        :type universal_id_type: str, optional
        """
        if namespace_id is not SENTINEL:
            self.namespace_id = namespace_id
        if universal_id is not SENTINEL:
            self.universal_id = universal_id
        if universal_id_type is not SENTINEL:
            self.universal_id_type = universal_id_type
        self._kwargs = kwargs




import requests

from requests.exceptions import Timeout
from typing import Generator, Optional, Tuple
from .base_handler import BaseHandler
from ...transport.request import Request
from ...transport.response import Response
from ...transport.api_error import ApiError


class HttpHandler(BaseHandler):
    """
    Handler for making HTTP requests.
    This handler sends the request to the specified URL and returns the response.

    :ivar int _timeout_in_seconds: The timeout for the HTTP request in seconds.
    """

    def __init__(self, timeout=60000):
        """
        Initialize a new instance of HttpHandler.
        """
        super().__init__()
        self._timeout_in_seconds = timeout / 1000

    def handle(
        self, request: Request
    ) -> Tuple[Optional[Response], Optional[Exception]]:
        """
        Send the request to the specified URL and return the response.

        :param Request request: The request to send.
        :return: The response and any error that occurred.
        :rtype: Tuple[Optional[Response], Optional[Exception]]
        """
        try:
            request_args = self._get_request_data(request)

            result = requests.request(
                request.method,
                request.url,
                headers=request.headers,
                timeout=self._timeout_in_seconds,
                **request_args,
            )
            response = Response(result)

            if response.status >= 400:
                if response.status in request.errors and isinstance(
                    response.body, dict
                ):
                    error_model_class = request.errors[response.status]
                    error = error_model_class(**response.body)
                    if "message" not in response.body:
                        error.message = (
                            f"{response.status} error in request to: {request.url}"
                        )
                    error.status = response.status
                    error.response = response

                    return None, error

                return None, ApiError(
                    message=f"{response.status} error in request to: {request.url}",
                    status=response.status,
                    response=response,
                )

            return response, None
        except Timeout:
            return None, ApiError("Request timed out", status=408)

    def stream(
        self, request: Request
    ) -> Generator[Tuple[Optional[Response], Optional[Exception]], None, None]:
        try:
            request_args = self._get_request_data(request)

            result = requests.request(
                request.method,
                request.url,
                headers=request.headers,
                timeout=self._timeout_in_seconds,
                stream=True,
                **request_args,
            )

            if result.status_code >= 400:
                response = Response(result)
                yield (
                    None,
                    ApiError(
                        message=f"{response.status} error in request to: {request.url}",
                        status=response.status,
                        response=response,
                    ),
                )

            else:
                for chunk in result.iter_content(chunk_size=8192):
                    for response in Response.from_chunk(result, chunk):
                        yield response, None

        except Timeout:
            yield None, ApiError("Request timed out", status=408)

    def _get_request_data(self, request: Request) -> dict:
        """
        Get the request arguments based on the request headers and data.

        :param Request request: The request object.
        :return: The request arguments.
        :rtype: dict
        """
        headers = request.headers or {}
        data = request.body or {}
        content_type = headers.get("Content-Type", "application/json")

        if request.method == "GET" and not data:
            return {}

        if content_type.startswith("application/") and "json" in content_type:
            return {"json": data}

        if "multipart/form-data" in content_type:
            headers.pop("Content-Type", None)
            files, form_data = {}, {}
            for key, value in data.items():
                if isinstance(value, bytes):
                    files[key] = (key, value, "application/octet-stream")
                else:
                    form_data[key] = value
            return {"files": files, "data": form_data}

        return {"data": data}




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .account_group_account_expression import (
    AccountGroupAccountExpression,
    AccountGroupAccountExpressionGuard,
)
from .account_group_account_simple_expression import AccountGroupAccountSimpleExpression
from .account_group_account_grouping_expression import (
    AccountGroupAccountGroupingExpression,
)


@JsonMap({})
class AccountGroupAccountQueryConfigQueryFilter(BaseModel):
    """AccountGroupAccountQueryConfigQueryFilter

    :param expression: expression
    :type expression: AccountGroupAccountExpression
    """

    def __init__(self, expression: AccountGroupAccountExpression, **kwargs):
        """AccountGroupAccountQueryConfigQueryFilter

        :param expression: expression
        :type expression: AccountGroupAccountExpression
        """
        self.expression = AccountGroupAccountExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class AccountGroupAccountQueryConfig(BaseModel):
    """AccountGroupAccountQueryConfig

    :param query_filter: query_filter
    :type query_filter: AccountGroupAccountQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: AccountGroupAccountQueryConfigQueryFilter, **kwargs
    ):
        """AccountGroupAccountQueryConfig

        :param query_filter: query_filter
        :type query_filter: AccountGroupAccountQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, AccountGroupAccountQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "component_id": "componentId",
        "created_by": "createdBy",
        "created_date": "createdDate",
        "environment_map_extension_id": "environmentMapExtensionId",
        "modified_by": "modifiedBy",
        "modified_date": "modifiedDate",
    }
)
class EnvironmentMapExtensionUserDefinedFunctionSummary(BaseModel):
    """EnvironmentMapExtensionUserDefinedFunctionSummary

    :param component_id: The ID of the component., defaults to None
    :type component_id: str, optional
    :param created_by: The email address of the user who created the component., defaults to None
    :type created_by: str, optional
    :param created_date: The creation date of the component., defaults to None
    :type created_date: str, optional
    :param deleted: Indicates the deletion of a component. A true value indicates a deleted status, whereas a false value indicates an active status., defaults to None
    :type deleted: bool, optional
    :param environment_map_extension_id: The system-generated ID of a environment map extension within a specific environment., defaults to None
    :type environment_map_extension_id: str, optional
    :param modified_by: The email address of the user who modified the component., defaults to None
    :type modified_by: str, optional
    :param modified_date: The modification date of the component., defaults to None
    :type modified_date: str, optional
    :param name: The user-defined name given to the component., defaults to None
    :type name: str, optional
    """

    def __init__(
        self,
        component_id: str = SENTINEL,
        created_by: str = SENTINEL,
        created_date: str = SENTINEL,
        deleted: bool = SENTINEL,
        environment_map_extension_id: str = SENTINEL,
        modified_by: str = SENTINEL,
        modified_date: str = SENTINEL,
        name: str = SENTINEL,
        **kwargs
    ):
        """EnvironmentMapExtensionUserDefinedFunctionSummary

        :param component_id: The ID of the component., defaults to None
        :type component_id: str, optional
        :param created_by: The email address of the user who created the component., defaults to None
        :type created_by: str, optional
        :param created_date: The creation date of the component., defaults to None
        :type created_date: str, optional
        :param deleted: Indicates the deletion of a component. A true value indicates a deleted status, whereas a false value indicates an active status., defaults to None
        :type deleted: bool, optional
        :param environment_map_extension_id: The system-generated ID of a environment map extension within a specific environment., defaults to None
        :type environment_map_extension_id: str, optional
        :param modified_by: The email address of the user who modified the component., defaults to None
        :type modified_by: str, optional
        :param modified_date: The modification date of the component., defaults to None
        :type modified_date: str, optional
        :param name: The user-defined name given to the component., defaults to None
        :type name: str, optional
        """
        if component_id is not SENTINEL:
            self.component_id = component_id
        if created_by is not SENTINEL:
            self.created_by = created_by
        if created_date is not SENTINEL:
            self.created_date = created_date
        if deleted is not SENTINEL:
            self.deleted = deleted
        if environment_map_extension_id is not SENTINEL:
            self.environment_map_extension_id = environment_map_extension_id
        if modified_by is not SENTINEL:
            self.modified_by = modified_by
        if modified_date is not SENTINEL:
            self.modified_date = modified_date
        if name is not SENTINEL:
            self.name = name
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .tracked_field import TrackedField


@JsonMap({"tracked_field": "trackedField"})
class TrackedFields(BaseModel):
    """Displays all the custom tracked fields from this document.

    :param tracked_field: tracked_field, defaults to None
    :type tracked_field: List[TrackedField], optional
    """

    def __init__(self, tracked_field: List[TrackedField] = SENTINEL, **kwargs):
        """Displays all the custom tracked fields from this document.

        :param tracked_field: tracked_field, defaults to None
        :type tracked_field: List[TrackedField], optional
        """
        if tracked_field is not SENTINEL:
            self.tracked_field = self._define_list(tracked_field, TrackedField)
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..component_environment_attachment import ComponentEnvironmentAttachmentService
from ...models import (
    ComponentEnvironmentAttachment,
    ComponentEnvironmentAttachmentQueryResponse,
    ComponentEnvironmentAttachmentQueryConfig,
)


class ComponentEnvironmentAttachmentServiceAsync(ComponentEnvironmentAttachmentService):
    """
    Async Wrapper for ComponentEnvironmentAttachmentServiceAsync
    """

    def create_component_environment_attachment(
        self, request_body: ComponentEnvironmentAttachment = None
    ) -> Awaitable[Union[ComponentEnvironmentAttachment, str]]:
        return to_async(super().create_component_environment_attachment)(request_body)

    def query_component_environment_attachment(
        self, request_body: ComponentEnvironmentAttachmentQueryConfig = None
    ) -> Awaitable[Union[ComponentEnvironmentAttachmentQueryResponse, str]]:
        return to_async(super().query_component_environment_attachment)(request_body)

    def query_more_component_environment_attachment(
        self, request_body: str
    ) -> Awaitable[Union[ComponentEnvironmentAttachmentQueryResponse, str]]:
        return to_async(super().query_more_component_environment_attachment)(
            request_body
        )

    def delete_component_environment_attachment(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_component_environment_attachment)(id_)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..account_cloud_attachment_quota import AccountCloudAttachmentQuotaService
from ...models import (
    AccountCloudAttachmentQuota,
    AccountCloudAttachmentQuotaBulkResponse,
    AccountCloudAttachmentQuotaBulkRequest,
)


class AccountCloudAttachmentQuotaServiceAsync(AccountCloudAttachmentQuotaService):
    """
    Async Wrapper for AccountCloudAttachmentQuotaServiceAsync
    """

    def create_account_cloud_attachment_quota(
        self, request_body: AccountCloudAttachmentQuota = None
    ) -> Awaitable[Union[AccountCloudAttachmentQuota, str]]:
        return to_async(super().create_account_cloud_attachment_quota)(request_body)

    def get_account_cloud_attachment_quota(
        self, id_: str
    ) -> Awaitable[Union[AccountCloudAttachmentQuota, str]]:
        return to_async(super().get_account_cloud_attachment_quota)(id_)

    def update_account_cloud_attachment_quota(
        self, id_: str, request_body: AccountCloudAttachmentQuota = None
    ) -> Awaitable[Union[AccountCloudAttachmentQuota, str]]:
        return to_async(super().update_account_cloud_attachment_quota)(
            id_, request_body
        )

    def delete_account_cloud_attachment_quota(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_account_cloud_attachment_quota)(id_)

    def bulk_account_cloud_attachment_quota(
        self, request_body: AccountCloudAttachmentQuotaBulkRequest = None
    ) -> Awaitable[Union[AccountCloudAttachmentQuotaBulkResponse, str]]:
        return to_async(super().bulk_account_cloud_attachment_quota)(request_body)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .operation import Operation


@JsonMap({})
class Operations(BaseModel):
    """Operations

    :param operation: operation, defaults to None
    :type operation: List[Operation], optional
    """

    def __init__(self, operation: List[Operation] = SENTINEL, **kwargs):
        """Operations

        :param operation: operation, defaults to None
        :type operation: List[Operation], optional
        """
        if operation is not SENTINEL:
            self.operation = self._define_list(operation, Operation)
        self._kwargs = kwargs




import re
import operator
from typing import List, Union, Type, Any, TypeVar, Optional
from enum import Enum
from .one_of_base_model import OneOfBaseModel
from .sentinel import SENTINEL

T = TypeVar("T")


class BaseModel:
    """
    A base class that most of the models in the SDK inherited from.
    """

    def __init__(self):
        pass

    def _define_object(self, input_data: Any, input_class: Type[T]) -> Optional[T]:
        """
        Check if the input data is an instance of the input class and return the input data if it is.
        Otherwise, return an instance of the input class.

        :param input_data: The input data to be checked.
        :param input_class: The class that the input data should be an instance of.
        :return: The input data if it is an instance of input_class, otherwise an instance of input_class.
        :rtype: object
        """
        if input_data is None or input_data is SENTINEL:
            return None
        elif isinstance(input_data, input_class):
            return input_data
        else:
            return input_class._unmap(input_data)

    def _define_list(
        self, input_data: Optional[List[Any]], list_class: Type[T]
    ) -> Optional[List[T]]:
        """
        Create a list of instances of a specified class from input data.
        :param input_data: The input data to be transformed into a list of instances.
        :param list_class: The class that each instance in the list should be an instance of.
        :return: A list of instances of list_class.
        :rtype: list
        """

        if input_data is None or input_data is SENTINEL:
            return None

        result: List[T] = []
        for item in input_data:
            if hasattr(list_class, "__args__") and len(list_class.__args__) > 0:
                class_list = self.__create_class_map(list_class)
                OneOfBaseModel.class_list = class_list
                result.append(OneOfBaseModel.return_one_of(item))
            elif issubclass(list_class, Enum):
                result.append(
                    self._enum_matching(item, list_class.list(), list_class.__name__)
                )
            elif isinstance(item, list_class):
                result.append(item)
            elif isinstance(item, dict):
                result.append(list_class._unmap(item))
            else:
                result.append(list_class(item))
        return result

    def _define_str(
        self,
        variable_name: str,
        value: Optional[Any],
        nullable=False,
        pattern: Optional[str] = None,
        min_length: Optional[int] = None,
        max_length: Optional[int] = None,
    ) -> Optional[str]:
        """
        Check if a string value is within the specified boundaries and matches a regex pattern.
        Return the value if it is, otherwise raise a ValueError.

        :param variable_name: The variable name.
        :type variable_name: str
        :param value: The value to be checked.
        :type value: str
        :param nullable: Whether the value can be None.
        :type nullable: bool
        :param pattern: The regex pattern to match.
        :type pattern: str
        :param min_length: The minimum length of the value.
        :type min_length: int
        :param max_length: The maximum length of the value.
        :type max_length: int
        :param inclusive_lenght_boundary: Whether the length boundaries are inclusive.
        :type inclusive_lenght_boundary: bool

        :raises ValueError: If the value is not within the boundaries or does not match the pattern.

        :return: The value if it is within the boundaries and matches the pattern.
        :rtype: str
        """
        if value is None and not nullable:
            raise ValueError(f"{variable_name} cannot be null.")
        if value is None or value is SENTINEL:
            return None

        if pattern:
            self._pattern_matching(value, pattern, variable_name)

        if min_length is not None:
            operation = operator.ge
            self._boundary_check(variable_name, len(value), min_length, operation, str)

        if max_length is not None:
            operation = operator.le
            self._boundary_check(variable_name, len(value), max_length, operation, str)

        return value

    def _define_bool(
        self,
        variable_name: str,
        value: Optional[Union[bool, str]],
        nullable=False,
    ) -> Optional[bool]:
        """
        Convert string boolean values to actual booleans.
        Return the value if it is, otherwise raise a ValueError.

        :param variable_name: The variable name.
        :type variable_name: str
        :param value: The value to be checked and converted.
        :type value: Union[bool, str]
        :param nullable: Whether the value can be None.
        :type nullable: bool

        :raises ValueError: If the value cannot be converted to boolean.

        :return: The boolean value.
        :rtype: bool
        """
        if value is None and not nullable:
            raise ValueError(f"{variable_name} cannot be null.")
        if value is None or value is SENTINEL:
            return None

        # Handle string boolean values from XML
        if isinstance(value, str):
            if value.lower() == 'true':
                return True
            elif value.lower() == 'false':
                return False
            else:
                raise ValueError(f"Invalid boolean string for {variable_name}: expected 'true' or 'false', received '{value}'")
        
        # Handle actual boolean values
        if isinstance(value, bool):
            return value
        
        # Try to convert other types
        return bool(value)

    def _define_number(
        self,
        variable_name: str,
        value: Optional[Union[float, int]],
        nullable=False,
        ge: Optional[Union[float, int]] = None,
        gt: Optional[Union[float, int]] = None,
        le: Optional[Union[float, int]] = None,
        lt: Optional[Union[float, int]] = None,
    ) -> Optional[Union[float, int]]:
        """
        Check if a number value is within the specified boundaries.
        Return the value if it is, otherwise raise a ValueError.

        :param variable_name: The variable name.
        :type variable_name: str
        :param value: The value to be checked.
        :type value: Union[float, int]
        :param nullable: Whether the value can be None.
        :type nullable: bool
        :param ge: The minimum value of the number (inclusive). The value must be greater than or equal to this.
        :type ge: Union[float, int]
        :param gt: The minimum value of the number (exclusive). The value must be greater than this.
        :type gt: Union[float, int]
        :param le: The maximum value of the number (inclusive). The value must be less than or equal to this.
        :type le: Union[float, int]
        :param lt: The maximum value of the number (exclusive). The value must be less than this.
        :type lt: Union[float, int]

        :raises ValueError: If the value is not within the boundaries.

        :return: The value if it is within the boundaries.
        :rtype: Union[float, int]
        """
        if value is None and not nullable:
            raise ValueError(f"{variable_name} cannot be null.")

        if value is None or value is SENTINEL:
            return None

        if ge is not None:
            operation = operator.ge
            self._boundary_check(variable_name, value, ge, operation, float)

        if gt is not None:
            operation = operator.gt
            self._boundary_check(variable_name, value, gt, operation, float)

        if le is not None:
            operation = operator.le
            self._boundary_check(variable_name, value, le, operation, float)

        if lt is not None:
            operation = operator.lt
            self._boundary_check(variable_name, value, lt, operation, float)

        return value

    def _boundary_check(
        self,
        variable_name: str,
        value: float,
        boundary: float,
        operation: operator,
        value_type: Union[str, float, int],
    ) -> Optional[str]:
        """
        Checks if a value is within the specified boundaries and returns the value if it is.

        :param variable_name: The variable name.
        :type variable_name: str
        :param value: The value to be checked.
        :type value: Union[str, float, int]
        :param boundary: The boundary value to be checked against.
        :type boundary: float
        :param operator: The operator to be used for the boundary check.
        :type operator: operator
        :param type: The type of the value.
        :type type: Union[str, float, int]

        :raises ValueError: If the value is not within the boundaries.

        :return: The value if it is within the boundaries.
        :rtype: Union[str, float, int]
        """
        reference = "length" if value_type is str else "value"
        if not operation(value, boundary):
            raise ValueError(
                f"Invalid {reference} for {variable_name}: must satisfy the condition {operation.__name__} {boundary}, received {value}"
            )

        return value

    def _pattern_matching(
        self, value: Optional[str], pattern: str, variable_name: str
    ) -> Optional[str]:
        """
        Checks if a value matches a regex pattern and returns the value if there's a match.

        :param value: The value to be checked.
        :type value: str
        :param pattern: The regex pattern.
        :type pattern: str
        :param variable_name: The variable name.
        :type variable_name: str
        :return: The value if it matches the pattern.
        :rtype: str
        :raises ValueError: If the value does not match the pattern.
        """
        if value is None:
            return None

        if re.match(r"{}".format(pattern), value):
            return value
        else:
            raise ValueError(
                f"Invalid value for {variable_name}: must match {pattern}, received {value}"
            )

    def _enum_matching(
        self, value: Union[str, Enum], enum_values: List[str], variable_name: str
    ) -> Union[str, Enum]:
        """
        Checks if a value (str or enum) matches the required enum values and returns the value if there's a match.

        :param value: The value to be checked.
        :type value: Union[str, Enum]
        :param enum_values: The list of valid enum values.
        :type enum_values: List[str]
        :param variable_name: The variable name.
        :type variable_name: str
        :return: The value if it matches one of the enum values.
        :rtype: Union[str, Enum]
        :raises ValueError: If the value does not match any of the enum values.
        """
        if value is None:
            return None

        str_value = value.value if isinstance(value, Enum) else value
        if str_value in enum_values:
            return value
        else:
            raise ValueError(
                f"Invalid value for {variable_name}: must match one of {enum_values}, received {value}"
            )

    def _get_representation(self, level: int = 0) -> str:
        """
        Get a string representation of the model.

        :param int level: The indentation level.
        :return: A string representation of the model.
        """
        indent = "    " * level
        representation_lines = []

        for attr, value in vars(self).items():
            if value is not None:
                value_representation = (
                    value._get_representation(level + 1)
                    if hasattr(value, "_get_representation")
                    else repr(value)
                )
                representation_lines.append(
                    f"{indent}    {attr}={value_representation}"
                )

        return (
            f"{self.__class__.__name__}(\n"
            + ",\n".join(representation_lines)
            + f"\n{indent})"
        )

    def __str__(self):
        return self._get_representation()

    def __repr__(self):
        return self._get_representation()

    def __create_class_map(self, union_type):
        """
        Create a dictionary that maps class names to the actual classes in a Union type.

        :param union_type: The Union type to create a class map for.
        :return: A dictionary mapping class names to classes.
        :rtype: dict
        """
        class_map = {}
        for arg in union_type.__args__:
            if arg.__name__:
                class_map[arg.__name__] = arg
        return class_map




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class DataType(Enum):
    """An enumeration representing different categories.

    :cvar CHARACTER: "CHARACTER"
    :vartype CHARACTER: str
    :cvar DATETIME: "DATETIME"
    :vartype DATETIME: str
    :cvar FLOAT: "FLOAT"
    :vartype FLOAT: str
    :cvar INTEGER: "INTEGER"
    :vartype INTEGER: str
    """

    CHARACTER = "CHARACTER"
    DATETIME = "DATETIME"
    FLOAT = "FLOAT"
    INTEGER = "INTEGER"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, DataType._member_map_.values()))


@JsonMap({"data_type": "dataType"})
class ScriptingParameter(BaseModel):
    """ScriptingParameter

    :param data_type: data_type, defaults to None
    :type data_type: DataType, optional
    :param index: index, defaults to None
    :type index: int, optional
    :param name: name, defaults to None
    :type name: str, optional
    """

    def __init__(
        self,
        data_type: DataType = SENTINEL,
        index: int = SENTINEL,
        name: str = SENTINEL,
        **kwargs
    ):
        """ScriptingParameter

        :param data_type: data_type, defaults to None
        :type data_type: DataType, optional
        :param index: index, defaults to None
        :type index: int, optional
        :param name: name, defaults to None
        :type name: str, optional
        """
        if data_type is not SENTINEL:
            self.data_type = self._enum_matching(
                data_type, DataType.list(), "data_type"
            )
        if index is not SENTINEL:
            self.index = index
        if name is not SENTINEL:
            self.name = name
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .process_schedule_status_expression import (
    ProcessScheduleStatusExpression,
    ProcessScheduleStatusExpressionGuard,
)
from .process_schedule_status_simple_expression import (
    ProcessScheduleStatusSimpleExpression,
)
from .process_schedule_status_grouping_expression import (
    ProcessScheduleStatusGroupingExpression,
)


@JsonMap({})
class ProcessScheduleStatusQueryConfigQueryFilter(BaseModel):
    """ProcessScheduleStatusQueryConfigQueryFilter

    :param expression: expression
    :type expression: ProcessScheduleStatusExpression
    """

    def __init__(self, expression: ProcessScheduleStatusExpression, **kwargs):
        """ProcessScheduleStatusQueryConfigQueryFilter

        :param expression: expression
        :type expression: ProcessScheduleStatusExpression
        """
        self.expression = ProcessScheduleStatusExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ProcessScheduleStatusQueryConfig(BaseModel):
    """ProcessScheduleStatusQueryConfig

    :param query_filter: query_filter
    :type query_filter: ProcessScheduleStatusQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: ProcessScheduleStatusQueryConfigQueryFilter, **kwargs
    ):
        """ProcessScheduleStatusQueryConfig

        :param query_filter: query_filter
        :type query_filter: ProcessScheduleStatusQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ProcessScheduleStatusQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .hl7_connector_record_expression import (
    Hl7ConnectorRecordExpression,
    Hl7ConnectorRecordExpressionGuard,
)
from .hl7_connector_record_simple_expression import Hl7ConnectorRecordSimpleExpression
from .hl7_connector_record_grouping_expression import (
    Hl7ConnectorRecordGroupingExpression,
)


@JsonMap({})
class Hl7ConnectorRecordQueryConfigQueryFilter(BaseModel):
    """Hl7ConnectorRecordQueryConfigQueryFilter

    :param expression: expression
    :type expression: Hl7ConnectorRecordExpression
    """

    def __init__(self, expression: Hl7ConnectorRecordExpression, **kwargs):
        """Hl7ConnectorRecordQueryConfigQueryFilter

        :param expression: expression
        :type expression: Hl7ConnectorRecordExpression
        """
        self.expression = Hl7ConnectorRecordExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class Hl7ConnectorRecordQueryConfig(BaseModel):
    """Hl7ConnectorRecordQueryConfig

    :param query_filter: query_filter
    :type query_filter: Hl7ConnectorRecordQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: Hl7ConnectorRecordQueryConfigQueryFilter, **kwargs
    ):
        """Hl7ConnectorRecordQueryConfig

        :param query_filter: query_filter
        :type query_filter: Hl7ConnectorRecordQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, Hl7ConnectorRecordQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class X12ConnectorRecordSimpleExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar EQUALS: "EQUALS"
    :vartype EQUALS: str
    :cvar STARTSWITH: "STARTS_WITH"
    :vartype STARTSWITH: str
    :cvar BETWEEN: "BETWEEN"
    :vartype BETWEEN: str
    :cvar GREATERTHAN: "GREATER_THAN"
    :vartype GREATERTHAN: str
    :cvar GREATERTHANOREQUAL: "GREATER_THAN_OR_EQUAL"
    :vartype GREATERTHANOREQUAL: str
    :cvar LESSTHAN: "LESS_THAN"
    :vartype LESSTHAN: str
    :cvar LESSTHANOREQUAL: "LESS_THAN_OR_EQUAL"
    :vartype LESSTHANOREQUAL: str
    """

    EQUALS = "EQUALS"
    STARTSWITH = "STARTS_WITH"
    BETWEEN = "BETWEEN"
    GREATERTHAN = "GREATER_THAN"
    GREATERTHANOREQUAL = "GREATER_THAN_OR_EQUAL"
    LESSTHAN = "LESS_THAN"
    LESSTHANOREQUAL = "LESS_THAN_OR_EQUAL"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                X12ConnectorRecordSimpleExpressionOperator._member_map_.values(),
            )
        )


class X12ConnectorRecordSimpleExpressionProperty(Enum):
    """An enumeration representing different categories.

    :cvar EXECUTIONID: "executionId"
    :vartype EXECUTIONID: str
    :cvar ATOMID: "atomId"
    :vartype ATOMID: str
    :cvar DATEPROCESSED: "dateProcessed"
    :vartype DATEPROCESSED: str
    :cvar ID: "id"
    :vartype ID: str
    :cvar ACTIONTYPE: "actionType"
    :vartype ACTIONTYPE: str
    :cvar CONNECTORTYPE: "connectorType"
    :vartype CONNECTORTYPE: str
    :cvar CONNECTORNAME: "connectorName"
    :vartype CONNECTORNAME: str
    :cvar OPERATIONNAME: "operationName"
    :vartype OPERATIONNAME: str
    :cvar DOCUMENTINDEX: "documentIndex"
    :vartype DOCUMENTINDEX: str
    :cvar SUCCESSFUL: "successful"
    :vartype SUCCESSFUL: str
    :cvar SIZE: "size"
    :vartype SIZE: str
    :cvar ERRORMESSAGE: "errorMessage"
    :vartype ERRORMESSAGE: str
    :cvar ISAACKSTATUS: "isaAckStatus"
    :vartype ISAACKSTATUS: str
    :cvar ISAACKREPORT: "isaAckReport"
    :vartype ISAACKREPORT: str
    :cvar ACKSTATUS: "ackStatus"
    :vartype ACKSTATUS: str
    :cvar ACKREPORT: "ackReport"
    :vartype ACKREPORT: str
    :cvar ISACONTROL: "isaControl"
    :vartype ISACONTROL: str
    :cvar GSCONTROL: "gsControl"
    :vartype GSCONTROL: str
    :cvar STCONTROL: "stControl"
    :vartype STCONTROL: str
    :cvar FUNCTIONALID: "functionalID"
    :vartype FUNCTIONALID: str
    :cvar TRANSACTIONSET: "transactionSet"
    :vartype TRANSACTIONSET: str
    :cvar TESTINDICATOR: "testIndicator"
    :vartype TESTINDICATOR: str
    :cvar SENDERIDQUALIFIER: "senderIDQualifier"
    :vartype SENDERIDQUALIFIER: str
    :cvar SENDERID: "senderID"
    :vartype SENDERID: str
    :cvar RECEIVERIDQUALIFIER: "receiverIDQualifier"
    :vartype RECEIVERIDQUALIFIER: str
    :cvar RECEIVERID: "receiverID"
    :vartype RECEIVERID: str
    :cvar APPSENDERID: "appSenderID"
    :vartype APPSENDERID: str
    :cvar APPRECEIVERID: "appReceiverID"
    :vartype APPRECEIVERID: str
    :cvar STANDARDID: "standardID"
    :vartype STANDARDID: str
    :cvar STANDARD: "standard"
    :vartype STANDARD: str
    :cvar GSVERSION: "gsVersion"
    :vartype GSVERSION: str
    :cvar AGENCYCODE: "agencyCode"
    :vartype AGENCYCODE: str
    :cvar GSDATE: "gsDate"
    :vartype GSDATE: str
    :cvar GSTIME: "gsTime"
    :vartype GSTIME: str
    :cvar OUTBOUNDVALIDATIONSTATUS: "outboundValidationStatus"
    :vartype OUTBOUNDVALIDATIONSTATUS: str
    :cvar OUTBOUNDVALIDATIONREPORT: "outboundValidationReport"
    :vartype OUTBOUNDVALIDATIONREPORT: str
    """

    EXECUTIONID = "executionId"
    ATOMID = "atomId"
    DATEPROCESSED = "dateProcessed"
    ID = "id"
    ACTIONTYPE = "actionType"
    CONNECTORTYPE = "connectorType"
    CONNECTORNAME = "connectorName"
    OPERATIONNAME = "operationName"
    DOCUMENTINDEX = "documentIndex"
    SUCCESSFUL = "successful"
    SIZE = "size"
    ERRORMESSAGE = "errorMessage"
    ISAACKSTATUS = "isaAckStatus"
    ISAACKREPORT = "isaAckReport"
    ACKSTATUS = "ackStatus"
    ACKREPORT = "ackReport"
    ISACONTROL = "isaControl"
    GSCONTROL = "gsControl"
    STCONTROL = "stControl"
    FUNCTIONALID = "functionalID"
    TRANSACTIONSET = "transactionSet"
    TESTINDICATOR = "testIndicator"
    SENDERIDQUALIFIER = "senderIDQualifier"
    SENDERID = "senderID"
    RECEIVERIDQUALIFIER = "receiverIDQualifier"
    RECEIVERID = "receiverID"
    APPSENDERID = "appSenderID"
    APPRECEIVERID = "appReceiverID"
    STANDARDID = "standardID"
    STANDARD = "standard"
    GSVERSION = "gsVersion"
    AGENCYCODE = "agencyCode"
    GSDATE = "gsDate"
    GSTIME = "gsTime"
    OUTBOUNDVALIDATIONSTATUS = "outboundValidationStatus"
    OUTBOUNDVALIDATIONREPORT = "outboundValidationReport"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                X12ConnectorRecordSimpleExpressionProperty._member_map_.values(),
            )
        )


@JsonMap({})
class X12ConnectorRecordSimpleExpression(BaseModel):
    """X12ConnectorRecordSimpleExpression

    :param argument: argument, defaults to None
    :type argument: List[str], optional
    :param operator: operator
    :type operator: X12ConnectorRecordSimpleExpressionOperator
    :param property: property
    :type property: X12ConnectorRecordSimpleExpressionProperty
    """

    def __init__(
        self,
        operator: X12ConnectorRecordSimpleExpressionOperator,
        property: X12ConnectorRecordSimpleExpressionProperty,
        argument: List[str] = SENTINEL,
        **kwargs
    ):
        """X12ConnectorRecordSimpleExpression

        :param argument: argument, defaults to None
        :type argument: List[str], optional
        :param operator: operator
        :type operator: X12ConnectorRecordSimpleExpressionOperator
        :param property: property
        :type property: X12ConnectorRecordSimpleExpressionProperty
        """
        if argument is not SENTINEL:
            self.argument = argument
        self.operator = self._enum_matching(
            operator, X12ConnectorRecordSimpleExpressionOperator.list(), "operator"
        )
        self.property = self._enum_matching(
            property, X12ConnectorRecordSimpleExpressionProperty.list(), "property"
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .connector_simple_expression import ConnectorSimpleExpression
from .connector_grouping_expression import ConnectorGroupingExpression


class ConnectorExpressionGuard(OneOfBaseModel):
    class_list = {
        "ConnectorSimpleExpression": ConnectorSimpleExpression,
        "ConnectorGroupingExpression": ConnectorGroupingExpression,
    }


ConnectorExpression = Union[ConnectorSimpleExpression, ConnectorGroupingExpression]




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "component_id": "componentId",
        "component_name": "componentName",
        "component_type": "componentType",
        "current_version": "currentVersion",
        "latest_version": "latestVersion",
        "pending_version": "pendingVersion",
    }
)
class PublisherPackagedComponent(BaseModel):
    """PublisherPackagedComponent

    :param component_id: ID of the primary component in the packaged component., defaults to None
    :type component_id: str, optional
    :param component_name: Name of the primary component in the packaged component., defaults to None
    :type component_name: str, optional
    :param component_type: Component type of the primary component in the packaged component., defaults to None
    :type component_type: str, optional
    :param current_version: Packaged component version of the component that is currently released in this integration pack., defaults to None
    :type current_version: str, optional
    :param deleted: If true, the packaged component will be removed from the integration pack in the next release., defaults to None
    :type deleted: bool, optional
    :param latest_version: Latest packaged component version of the component that is available to be added to this integration pack., defaults to None
    :type latest_version: str, optional
    :param pending_version: Packaged component version of the component that will be included in the next release of this integration pack., defaults to None
    :type pending_version: str, optional
    """

    def __init__(
        self,
        component_id: str = SENTINEL,
        component_name: str = SENTINEL,
        component_type: str = SENTINEL,
        current_version: str = SENTINEL,
        deleted: bool = SENTINEL,
        latest_version: str = SENTINEL,
        pending_version: str = SENTINEL,
        **kwargs
    ):
        """PublisherPackagedComponent

        :param component_id: ID of the primary component in the packaged component., defaults to None
        :type component_id: str, optional
        :param component_name: Name of the primary component in the packaged component., defaults to None
        :type component_name: str, optional
        :param component_type: Component type of the primary component in the packaged component., defaults to None
        :type component_type: str, optional
        :param current_version: Packaged component version of the component that is currently released in this integration pack., defaults to None
        :type current_version: str, optional
        :param deleted: If true, the packaged component will be removed from the integration pack in the next release., defaults to None
        :type deleted: bool, optional
        :param latest_version: Latest packaged component version of the component that is available to be added to this integration pack., defaults to None
        :type latest_version: str, optional
        :param pending_version: Packaged component version of the component that will be included in the next release of this integration pack., defaults to None
        :type pending_version: str, optional
        """
        if component_id is not SENTINEL:
            self.component_id = component_id
        if component_name is not SENTINEL:
            self.component_name = component_name
        if component_type is not SENTINEL:
            self.component_type = component_type
        if current_version is not SENTINEL:
            self.current_version = current_version
        if deleted is not SENTINEL:
            self.deleted = deleted
        if latest_version is not SENTINEL:
            self.latest_version = latest_version
        if pending_version is not SENTINEL:
            self.pending_version = pending_version
        self._kwargs = kwargs




"""SDK top level package."""

__all__ = ["Boomi", "BoomiAsync", "Environment"]

def __getattr__(name):
    if name == "Boomi":
        from .sdk import Boomi as _Boomi
        return _Boomi
    if name == "BoomiAsync":
        from .sdk_async import BoomiAsync as _BoomiAsync
        return _BoomiAsync
    if name == "Environment":
        from .net.environment import Environment as _Environment
        return _Environment
    raise AttributeError(name)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .java_upgrade_options import JavaUpgradeOptions


@JsonMap({"java_upgrade_options": "JavaUpgradeOptions", "atom_id": "atomId"})
class JavaUpgrade(BaseModel):
    """JavaUpgrade

    :param java_upgrade_options: java_upgrade_options, defaults to None
    :type java_upgrade_options: JavaUpgradeOptions, optional
    :param atom_id: The unique ID assigned by the system to the container\<br /\> 1. Use the Runtime ID for Runtimes, Runtime clusters, and Runtime clouds found in the user interface by navigating to **Manage** \> **Runtime Management** and viewing the **Runtime Information** panel for a selected container. \<br /\> 2. Use the Gateway ID found in the user interface by navigating to **Configure Server** \> **Gateways** \> **click on a Gateway's name** \> **Gateway Information panel**. \<br /\> 3. Use the Broker ID found in the user interface by navigating to **Configure Server** \> **Authentication** \> **click on a Broker's name** \> **Broker Information panel**., defaults to None
    :type atom_id: str, optional
    """

    def __init__(
        self,
        java_upgrade_options: JavaUpgradeOptions = SENTINEL,
        atom_id: str = SENTINEL,
        **kwargs,
    ):
        """JavaUpgrade

        :param java_upgrade_options: java_upgrade_options, defaults to None
        :type java_upgrade_options: JavaUpgradeOptions, optional
        :param atom_id: The unique ID assigned by the system to the container\<br /\> 1. Use the Runtime ID for Runtimes, Runtime clusters, and Runtime clouds found in the user interface by navigating to **Manage** \> **Runtime Management** and viewing the **Runtime Information** panel for a selected container. \<br /\> 2. Use the Gateway ID found in the user interface by navigating to **Configure Server** \> **Gateways** \> **click on a Gateway's name** \> **Gateway Information panel**. \<br /\> 3. Use the Broker ID found in the user interface by navigating to **Configure Server** \> **Authentication** \> **click on a Broker's name** \> **Broker Information panel**., defaults to None
        :type atom_id: str, optional
        """
        if java_upgrade_options is not SENTINEL:
            self.java_upgrade_options = self._define_object(
                java_upgrade_options, JavaUpgradeOptions
            )
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .environment_role import EnvironmentRole


@JsonMap({"number_of_results": "numberOfResults", "query_token": "queryToken"})
class EnvironmentRoleQueryResponse(BaseModel):
    """EnvironmentRoleQueryResponse

    :param number_of_results: number_of_results, defaults to None
    :type number_of_results: int, optional
    :param query_token: query_token, defaults to None
    :type query_token: str, optional
    :param result: result, defaults to None
    :type result: List[EnvironmentRole], optional
    """

    def __init__(
        self,
        number_of_results: int = SENTINEL,
        query_token: str = SENTINEL,
        result: List[EnvironmentRole] = SENTINEL,
        **kwargs,
    ):
        """EnvironmentRoleQueryResponse

        :param number_of_results: number_of_results, defaults to None
        :type number_of_results: int, optional
        :param query_token: query_token, defaults to None
        :type query_token: str, optional
        :param result: result, defaults to None
        :type result: List[EnvironmentRole], optional
        """
        if number_of_results is not SENTINEL:
            self.number_of_results = number_of_results
        if query_token is not SENTINEL:
            self.query_token = query_token
        if result is not SENTINEL:
            self.result = self._define_list(result, EnvironmentRole)
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .process_simple_expression import ProcessSimpleExpression
from .process_grouping_expression import ProcessGroupingExpression


class ProcessExpressionGuard(OneOfBaseModel):
    class_list = {
        "ProcessSimpleExpression": ProcessSimpleExpression,
        "ProcessGroupingExpression": ProcessGroupingExpression,
    }


ProcessExpression = Union[ProcessSimpleExpression, ProcessGroupingExpression]




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..release_integration_pack_status import ReleaseIntegrationPackStatusService
from ...models import (
    ReleaseIntegrationPackStatus,
    ReleaseIntegrationPackStatusBulkResponse,
    ReleaseIntegrationPackStatusBulkRequest,
)


class ReleaseIntegrationPackStatusServiceAsync(ReleaseIntegrationPackStatusService):
    """
    Async Wrapper for ReleaseIntegrationPackStatusServiceAsync
    """

    def get_release_integration_pack_status(
        self, id_: str
    ) -> Awaitable[Union[ReleaseIntegrationPackStatus, str]]:
        return to_async(super().get_release_integration_pack_status)(id_)

    def bulk_release_integration_pack_status(
        self, request_body: ReleaseIntegrationPackStatusBulkRequest = None
    ) -> Awaitable[Union[str, ReleaseIntegrationPackStatusBulkResponse]]:
        return to_async(super().bulk_release_integration_pack_status)(request_body)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"runtime_id": "runtimeId"})
class RuntimeRestartRequest(BaseModel):
    """RuntimeRestartRequest

    :param message: message, defaults to None
    :type message: str, optional
    :param runtime_id: A unique ID for the runtime. Cloud attachments cannot be restarted., defaults to None
    :type runtime_id: str, optional
    """

    def __init__(self, message: str = SENTINEL, runtime_id: str = SENTINEL, **kwargs):
        """RuntimeRestartRequest

        :param message: message, defaults to None
        :type message: str, optional
        :param runtime_id: A unique ID for the runtime. Cloud attachments cannot be restarted., defaults to None
        :type runtime_id: str, optional
        """
        if message is not SENTINEL:
            self.message = message
        if runtime_id is not SENTINEL:
            self.runtime_id = runtime_id
        self._kwargs = kwargs
